/*
 * CApproximation.h
 *
 *  Created on: Dec 2, 2013
 *      Author: jono and miro
 */

#ifndef CAPPROXIMATION_H_
#define CAPPROXIMATION_H_

#include "CPolynomial.h"
#include "CCube.h"
#include "CComplex.h"
#include "CFiltration.h"
#include <vector>
#include <iostream>
#include <stack> 



class CApproximation {
private:
  /* We store the the maximum allowable depth as a global constant for the class */
  unsigned int Maximum_Depth;
  /* We store a function and a CW complex */
  CPolynomial poly_;
  double Convert_Threshold(int integer_threshold, double bottom_threshold, double step_size, unsigned int max_steps, bool sublevel);
  

public:
	CApproximation();                                    // Initializor 
	CApproximation(const CApproximation& approx);        //  Copy constructor
	virtual ~CApproximation(void);                       // Destructor

	
  bool Verified_Approximation( double threshold, bool sublevel, CComplex & complex);
  void Intersect( CComplex & complex_A, CComplex & complex_B); // Modifies complex B
  void Verified_Filtration(double bottom_threshold, double step_size, unsigned int max_steps, bool sublevel, CFiltration & filter);

  void Modify_Output(double bottom_threshold, double step_size, unsigned int max_steps, bool sublevel);


  /* Defines what the maximum number of subdivisions of the unit cube are used in the main algorithm */
  void Define_Maximum_Depth(unsigned int New_Depth){
	  Maximum_Depth = New_Depth;
  };
  /* Define how many times a cube is subdivided when computing interval arithmetic */
  void Define_Interval_Subdivisions(unsigned int New_Subdivisions){
	  poly_.Define_Interval_Subdivisions(New_Subdivisions);
  }


};

#endif /* CAPPROXIMATION_H_ */
