/*
 * CNode.cpp
 *
 *  Created on: Dec 6, 2013
 *      Author: Jono
 */

#include "CNode.h"


/*
***************************
******   PRIVATE    *******
*************************** 
*/



/*
***************************
******   BIG THREE    *******
*************************** 
*/

//  Generic Constructor
CNode::CNode(){
}

/* Copy constructor */
CNode::CNode( const CNode& node ){
  parent_       = node.parent_;
  children_     = node.children_;
  faces_        = node.faces_;
  
  birth_time_   = node.birth_time_   ;
  export_order_ = node.export_order_ ;
  
}

/*
Destructor
  This will destroy the node, and all of its children
  */
CNode::~CNode(){
  deletion();
}

/*
***************************
******   PUBLIC     *******
*************************** 
*/


/*
This function checks to see whether the node is a leaf (returns true) or not (returns false ).
*/
bool CNode::leaf_check(void){
  if( children_.size() >0 )
    return false;
  else
    return true;
}


void CNode::split(void){
  for(int i = 0 ; i < 4 ; i++){
    CNode * child = new CNode;
    (*child).parent_ = this; 
    children_.push_back(child);
  }
}


/*
NOTE THis only works if faces_ is empty to begin with. 

The faces of cube cooresponding to the node.
For a 2-cube, there are nine possible faces.
Indexing the faces proceeds as follows
  the first element is the single 2-cell
  the next elements are the 1-cells, ordered counter clockwise, starting with the bottom edge
  The verticies are ordered as follows:: (0,0) ; (0,1) ; (1,0) ; (1,1) 
*/
void CNode::store_faces(std::vector< bool > input_faces ){
  for(int i =0 ; i<9 ; i++){
    faces_.push_back(input_faces[i]); 
  }
}


/*
Deletes all of node's children, assuming the node is not a leaf.
*/
void CNode::deletion( void ){
  /* If the node is a leaf, don't do anything */ 
  if( !(leaf_check()) ){
    /* Iterate through all of the children of the node */
    for( int i = 0 ; i< 4  ; i++){
      /* Delete all of the children of the node. */
      delete children_[i];
    }
    children_.clear();
  }
}

/*
Return a given children
*/
CNode * CNode::get_child( unsigned int child ){
  if( child > 3)
    std::cout << " CRITICAL ERROR \n";
  return children_[child];
}

/*
Return a given parent
*/
CNode * CNode::get_parent( void ){
  return parent_;
}


/*
 Returns the order in which the given node was born. 
  WARNING ::::  This will have segmentation faults if you pass the root of a complex.
*/
unsigned int CNode::child_number( void  ){
  // We iterate through the four children of the parent
  for(unsigned int i = 0; i < 4 ;i++){
    // If the i^th child is equal to the input child, then we return "i".
    if( (*parent_).get_child(i) == this)
      return i;
  }
}


/*
Checks to see if a given face is stored in the node.
*/
bool CNode::Check_Cell( unsigned int face_number){
  return faces_[ face_number ];
}

void CNode::PrintFaces(void ){
  if( leaf_check() ){
    if( faces_.empty() ){
      std::cout << "NODE HAS NEITHER FACES NOR CHILDREN \n";
    }
    else{
      std::cout << "Faces :: ";
      for(int i =0; i < 9 ; i++){
        if(faces_[i]){
          std::cout << i ;
        }
      }
      std::cout << "\n";
    }
  }
  else{
    std::cout << "Node is a branch \n";
  }
}

/*
This function is called inside of CFiltration::Refine. 
Subdivides the node and copies birth times.
Assume that faces_ is all ones. copies that info too
*/
void CNode::Filtered_Subdivide( void ){
  /* We first split the node */
  split();
  /* We create a boolean vector, of size 9, all entries are true */
  std::vector < bool > all_cells( 9,true);
  /* We add all of the cells to the children nodes */
  for(int i = 0 ; i < 4 ; i++){
    (*children_[i]).store_faces(all_cells);
    /* We resize the birth vector of  the children */
    (*children_[i]).ResizeBirth();
  }
  /* This next bit is going to get pretty ugly */
  int local_birth;
  int local_cell;
  
  
  /* We copy the birth information for square (cell 0) */
  local_cell = 0;
  local_birth = birth_time_[local_cell ];
  (*children_[0]).Set_Birth( local_cell, local_birth);
  (*children_[0]).Set_Birth( 2, local_birth);
  (*children_[0]).Set_Birth( 3, local_birth);
  (*children_[0]).Set_Birth( 8, local_birth);
  
  (*children_[1]).Set_Birth( local_cell, local_birth);
  (*children_[1]).Set_Birth( 3, local_birth);
  (*children_[1]).Set_Birth( 4, local_birth);
  (*children_[1]).Set_Birth( 6, local_birth);
  
  (*children_[2]).Set_Birth( local_cell, local_birth);
  (*children_[2]).Set_Birth( 1, local_birth);
  (*children_[2]).Set_Birth( 2, local_birth);
  (*children_[2]).Set_Birth( 7, local_birth);
  
  (*children_[3]).Set_Birth( local_cell, local_birth);
  (*children_[3]).Set_Birth( 1, local_birth);
  (*children_[3]).Set_Birth( 4, local_birth);
  (*children_[3]).Set_Birth( 5, local_birth);
  
  
  
  /* We copy the birth information for edge (cell 1) */
  local_cell = 1;
  local_birth = birth_time_[local_cell ];
  (*children_[0]).Set_Birth( local_cell, local_birth);
  (*children_[0]).Set_Birth( 7, local_birth);
  
  (*children_[1]).Set_Birth( local_cell, local_birth);
  (*children_[1]).Set_Birth( 5, local_birth);
  
  
  /* We copy the birth information for edge (cell 2) */
  local_cell = 2;
  local_birth = birth_time_[local_cell ];
  (*children_[1]).Set_Birth( local_cell, local_birth);
  (*children_[1]).Set_Birth( 8, local_birth);
  
  (*children_[3]).Set_Birth( local_cell, local_birth);
  (*children_[3]).Set_Birth( 7, local_birth);
  
  
  /* We copy the birth information for edge (cell 3) */
  local_cell = 3;
  local_birth = birth_time_[local_cell ];
  (*children_[2]).Set_Birth( local_cell, local_birth);
  (*children_[2]).Set_Birth( 8, local_birth);
  
  (*children_[3]).Set_Birth( local_cell, local_birth);
  (*children_[3]).Set_Birth( 6, local_birth);
  
  
  /* We copy the birth information for edge (cell 4) */
  local_cell = 4;
  local_birth = birth_time_[local_cell ];
  (*children_[0]).Set_Birth( local_cell, local_birth);
  (*children_[0]).Set_Birth( 6, local_birth);
  
  (*children_[2]).Set_Birth( local_cell, local_birth);
  (*children_[2]).Set_Birth( 5, local_birth);
  
  
  /* We copy the birth information for vertex (cell 5) */
  local_birth = birth_time_[5];
  (*children_[0]).Set_Birth( 5, local_birth);
  
  /* We copy the birth information for vertex (cell 6) */
  local_birth = birth_time_[6];
  (*children_[2]).Set_Birth( 6, local_birth);
  
  /* We copy the birth information for vertex (cell 7) */
  local_birth = birth_time_[7];
  (*children_[1]).Set_Birth( 7, local_birth);
  
  /* We copy the birth information for vertex (cell 8) */
  local_birth = birth_time_[8];
  (*children_[3]).Set_Birth( 8, local_birth);
  
  
  
  /* We clear the original vectors face_ and birth_time_ */
  faces_.clear();
  birth_time_.clear(); 
}
