/*
 * CNode.h
 *
 *  Created on: Dec 6, 2013
 *      Author: Jono
 */

#ifndef CNODE_H_
#define CNODE_H_


#include <vector>
#include <iostream>
#include <stack> 



class CNode {
  private:
    /* A pointer to the parent of the node */
    CNode* parent_;
    
    
    /* 
    A vector of pointers to the children of the node.
    In two dimensions, a node would have four children.
    This should be empty iff the node stores faces.
    
    The convention we use is::   
    * If it is empty the cube is the unit cube.
    * 0 - left bottom cube
    * 1 - right bottom cube
    * 2 - left top cube
    * 3 - right top cube     
    */
    std::vector< CNode* >  children_;
    
    
    // These functions are used in the Ascend function 
    
    
 
/* 
    The faces of cube cooresponding to the node.
    For a 2-cube, there are nine possible faces.
    Indexing the faces proceeds as follows
    the first element is the single 2-cell
    the next elements are the 1-cells, ordered counter clockwise, starting with the bottom edge
    The verticies are ordered as follows:: (0,0) ; (0,1) ; (1,0) ; (1,1) 
*/
    std::vector< bool >  faces_;
    
    
    std::vector < int > birth_time_;
    
    std::vector < int > export_order_ ;
    
  public:
    CNode();
    CNode( const CNode& node );
    ~CNode();
        bool leaf_check(void);
        void deletion(void);
        void split(void);
        void store_faces( std::vector< bool > input_faces  );
        void delete_face( int face_number){ faces_[ face_number ]=0;};
        CNode * get_child( unsigned int child );
        CNode * get_parent(void);
            // Returns the order in which the given node was born. 
        unsigned int child_number( void ) ;
        bool Check_Cell( unsigned int face_number);
        void PrintFaces(void );
        
        void ResizeBirth( void){ birth_time_.resize(9);};
        void ResizeExport( void){ export_order_.resize(9);};
        void  Set_Export( int face_number, int export_index ){ 
          export_order_[face_number ] = export_index;
        };
        void  Set_Birth( int face_number, int birthing_time ){ 
          birth_time_[face_number ] = birthing_time ;
        };
        
        int  Check_Birth(  int face_number){ return birth_time_[face_number ];};
        int  Check_Export( int face_number){ return export_order_[face_number ];};
        
        void Filtered_Subdivide( void );

        




};

#endif /* CNODE_H_ */
