#ifndef SGEOMETRICCUBE_H_
#define SGEOMETRICCUBE_H_

#include <boost/numeric/interval.hpp>
using namespace boost::numeric;
using namespace interval_lib;



/* In order to to use transcendental functions, such as Sine and Cosine, we define the type of interval arithmetic we use as follows */
typedef interval<double, policies<save_state<rounded_transc_std<double> >, checking_base<double> > > Interval;



#include<vector>
#include <iostream>

struct GeometricCube{
  unsigned int dimension_;
  std::vector< Interval > coordinates_;
  public:
    void Print(){
      std::cout << "X = [ "  << coordinates_[ 0 ].lower() << ", " << coordinates_[ 0 ].upper() << " ]   ";
      std::cout << "Y = [ "  << coordinates_[ 1 ].lower() << ", " << coordinates_[ 1 ].upper() << " ] \n";
    };
    std::vector< GeometricCube >  Subdivide(){ 
      /* We determine the widths of the x and the y coordinates */
      double x_width =  width(coordinates_[ 0 ]);
      double y_width =  width(coordinates_[ 1 ]);
      /* We make a list of the x- and y- intervals we want to take a product of */
      std::vector < Interval > x_sub_list;
      std::vector < Interval > y_sub_list; 
      /* If one of the widths is zero, then we do not want to subdivide, otherwise we do */ 
      if( x_width ==0){
        /* The x-coordinate is a single value */
        x_sub_list.push_back(coordinates_[ 0 ]);
      }
      else{
        /* We create the upper half and the lower half of the interval */
        Interval bottom_x = Interval(coordinates_[ 0 ].lower(), coordinates_[ 0 ].lower()+( x_width/2.0) );
        Interval top_x = Interval( coordinates_[ 0 ].lower()+( x_width/2.0) ,coordinates_[ 0 ].upper() );
        /* We add the two intervals to our list */
        x_sub_list.push_back( bottom_x );
        x_sub_list.push_back( top_x );
      }
      if( y_width ==0){
        /* The y-coordinate is a single value */
        y_sub_list.push_back(coordinates_[ 1 ]);
      }
      else{
        /* We create the upper half and the lower half of the interval */
        Interval bottom_y = Interval(coordinates_[ 1 ].lower(), coordinates_[ 1 ].lower()+( y_width/2.0) );
        Interval top_y = Interval( coordinates_[ 1 ].lower()+( y_width/2.0),coordinates_[ 1 ].upper() );
        /* We add the two intervals to our list */
        y_sub_list.push_back( bottom_y );
        y_sub_list.push_back( top_y );
      }
      /* We initialize the output vector */
      std::vector< GeometricCube > output;
      /* We initialize our coordinates vector for the output */
      std::vector < Interval > local_coordinates;
      
      /* We now combine the two subdivided lists */
      for( int i = 0 ; i < x_sub_list.size(); i ++){
        /* We add the x-interval */
        local_coordinates.push_back( x_sub_list[i] );
        for( int j = 0 ; j < y_sub_list.size(); j ++){
          /* We add the y-interval */
          local_coordinates.push_back( y_sub_list[j] );
          /* We create a new GeometricCube and add it to our output vector */
          GeometricCube local_output = { dimension_ , local_coordinates};
          output.push_back( local_output );
          /* We remove the last y-interval from the local_coordinates vector */
          local_coordinates.pop_back();
        }
        /* We remove the last x-interval from the local_coordinates vector */
        local_coordinates.pop_back();
      }

      return output;
    };
};

#endif /* SGEOMETRICCUBE_H_ */
