(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    364819,       6750]
NotebookOptionsPosition[    361915,       6654]
NotebookOutlinePosition[    362286,       6670]
CellTagsIndexPosition[    362243,       6667]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Tutorial 1: Getting Started", "Section",
 CellChangeTimes->{{3.6156444963118916`*^9, 3.615644502389239*^9}, {
  3.6156555533009233`*^9, 3.6156555563871*^9}}],

Cell["\<\
We are going to start by computing the persistent homology of the following \
function:\
\>", "Text",
 CellChangeTimes->{{3.61556175039701*^9, 3.615561767270975*^9}, {
  3.6157295604395423`*^9, 3.6157295639557433`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"20.0", 
    RowBox[{"(", 
     RowBox[{"y", "+", ".1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"y", "-", ".1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"y", "-", ".6"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"y", "-", ".8"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"y", "-", "1.05"}], ")"}]}], "-", 
   RowBox[{"10.0", 
    RowBox[{"(", 
     RowBox[{"x", "+", ".01"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "-", ".2"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "-", ".7"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "-", ".95"}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.615561588577754*^9, 3.615561593085012*^9}, {
  3.615564952834276*^9, 3.6155655637102165`*^9}, {3.61556559978428*^9, 
  3.6155656344942646`*^9}, {3.6155657864849586`*^9, 3.615565797912612*^9}, {
  3.615566765357947*^9, 3.6155667686401343`*^9}, {3.615567079474913*^9, 
  3.615567252894832*^9}, {3.615644657434107*^9, 3.61564466867375*^9}, {
  3.6156497670573606`*^9, 3.61564978366131*^9}}],

Cell[TextData[{
 "We can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ContourPlot function to get an idea of what this \
function looks like. \nHere we can see that the function is approximately \
between -0.4 and 0.2."
}], "Text",
 CellChangeTimes->{{3.6155617866310825`*^9, 3.6155618467625217`*^9}, {
   3.6155656139010873`*^9, 3.6155656236826468`*^9}, {3.615565834384698*^9, 
   3.6155658424771605`*^9}, 3.6155672032459927`*^9, {3.6155673351595373`*^9, 
   3.615567360272974*^9}, 3.6155692322160425`*^9, 3.6156446899789686`*^9, 
   3.617559276766098*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ContourPlot", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "Automatic"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6155672132455645`*^9, 3.6155672186478734`*^9}, {
  3.6155691664362803`*^9, 3.615569219040289*^9}, {3.6155695115980225`*^9, 
  3.61556951208405*^9}, {3.615644678586317*^9, 3.615644679287357*^9}, {
  3.6161623418478985`*^9, 3.616162344275038*^9}}],

Cell[BoxData[
 TemplateBox[{GraphicsBox[
    GraphicsComplexBox[CompressedData["
1:eJyMvQmczlX0P44iWbImsiQUZQtZohxJWYpQ2StLQpJIsguVkF3CkJ0Y62As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     "], {{{
        EdgeForm[], 
        RGBColor[0.148, 0.33, 0.54], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNlktoXVUUhve9N4/GPO/NuTe5JLlJBtWRohaTiK+mKQoiDuqkpWBpKppY
qbWBVhCx7URHTjqwoFLbxmAHBQcpFomJI0vAiujIB6gDHwlUmiq1UUj8P/91
wcFh77PPev7r32ud4ckXdx3Kp5R+01PQ83FLSr1bUppvSKlPB9tyKe3S+YdZ
SiPaL2k925XSOT3Nep/q9P45rQMhPyH5ocaUjuh9Z/I3dBbl6LDOXipY5vly
Sn/L3yOVlJ7olaz2qTul4UbL1+2jj95Aziu6uaaU2mXvKe3bpX/xtpRata7L
xjvan5KNnsijTXLfSH6vzgal/0x7SgfafbZVMTwmmXzJ8RPbpvysyf91PSN6
OiqWKZSsgy62+guOAXwm9H1J+/ukO9tkOXzVOvTemtJbemoFfyfmHQ32N6T3
V4vKXes1+ZnR+ZyeWs55XN3iXG7o28HmlP6ouD639FyS3N0V43JGdTlUdP7T
yilftk1s4584eOc79sD4dJwT2wyxRL12Nng/nLOPmYgHzIkBHpAH+ZELmA1E
XQYyc+iusrGphUwqmw8PZsaqFrnPBk74Qr4/zn+uOL5ftLb3pLSsfZvWR6vC
Wzp94smFDp/foVwW8j4HCziIHeoyr1g2mp3DPulual9SDF9L/5T2OcV1VBh9
KTy367yjxzE9ru+rZXPwoL6vaH9Ttp4tGzswhM/wF6xS5EoOxF/LzL/9Vctx
H8B2KHP9JqvWQ5/cB4J36I4r1k/yzuNytzGn/p+3mrtwC15cbrbd3Xp+0rOR
M2fh7u2N5ugHHcb/r17j9GvFOuhSj7xsjbW4tm83Wx/dkeA/d+fPimXOKfYx
nY12mVPgj338gB/66IIrfMJ+V2b8n66aJ4cjT2TPtNg+HNuMGoHJTOB5JPbg
xndqWZeBf5yT957InW+7Y48//Bbl/56qz18r+e7CmXovYAUfbHzVZB49pHp/
pHzuzHw3qCmcfy8zx/pVk2vdxvC4sLkp+RvUUtxYKbh2xPC+dJfbjclqwf2B
3N8IPsPrk9Kvan9Ca3fmPOHpVPBsf8m1RgZ/053uiXDx+27jeDTqA0+oI7Xr
C/4TOzmwh7NDwVtiWQ1skflPL2d+Y/uLsm1QH+4+PKHWcGguesJGnKMLhtQT
zreFz9nwuxi+yH2u1bjQi8BpJfrGQuzruM2GzEzFXDhftMxAzBd4iBzYXi37
nF7IfSOGQWF5oOR44f5y8HVvYAq2D2TmOBgSMziy/0643t9iTMnvesVxr2nd
F/ea+w0e+cgdrsExcNkT/YhcJiN+8qCvDgZ3t0X+5AI2YEQtzmf2PZuZj7uC
k9wD7lUKnBZCl/7xZqt7yFz0DepFDeszk1k6HHXfzMwj+Ptjxbo/kF/R/aWz
x/N2MbhKrEsR22bs6VHM1UsxF5j/Z+MfABzn45wZ1x//BL0xr/hGXMhij3nP
3L+ouH6Pnk/vZ75xF9DBPpynF3MnuZvrmXOcin8D1umQoS4XgqvkRL+kp9Nz
iWM4s0/OhmOOURv2Q7Enb/r5ROyfzIw/c4M7Tb+De992m/cvK+bPhN12yb4i
LO+tmgcnS86J+zAS3B6L/jke/1hgdStzD6GXoI+dNtk8VrEvZhS9m5xe7zGu
4EstmKn0yg2tLxSNwWrcX3zUuTYedfk0c69gttAT6C/0BfoafGPGUueJsL8u
W+/qX+RKj+fUjrB5ouQcyXU05g69h3+K0/+743PRK5AZi/nSGv9snRX/V/F/
tTViHom583DcHe7Q8ZLvF738n4iHuEbDJ31mLWYT9o8VjTf/rWA9GjL/Ajxv
T5Q=
           "]], 
          PolygonBox[CompressedData["
1:eJwlkz1IlXEUxp97S02v19t77/u+VxFJp7ZGKYJ0EIKIBqeagmzoesvI5br5
sdTU0hJUGGpOuQlFS02FkBGNFVRL0NCHFaEG2e/hDAee8z/nPOfzP3Th6vhU
UdIwsh+Zy6XHHVJfSVoAv6hL97qk0YL0HPyBt5vYdjulyUy6kkibB6S7+Gyj
N/ukRkX6g/0L+k4qPcL+cx92cH9VWidRnbfz6ENIL/gJ/gP2rUkbZekBOb6T
67h92qQO8l8mVxV9j/q2iSnB1UIvtkuH8ZlA/0rMGL63if8GPgXfCvYx+E/3
Su/Q7xD/mvgW9ofmL0RPH9G3kNVScBSp/3o9uDd4uwEu8HaL+Gn6EXgtDW5z
mOtQGr0t8vaWXN31iJ1wT9SfYP9H/Dr2CngRGSB2nBrvgxfo4Rz5PiFd1Zip
Z7uD/1QSM/FsXOMP8DJvKkZNZ4gfTaM3z9CzbCBv2oNzHr5fxDTxPQrfb/As
b2fboqaDxFao92UpZtYDruCzRv6T7KwH3MiiFu/kEngmj1490/fkGsnjNlzT
iTx6cC/OMQd+lcVtuOdN8JEsdu+b8G2sUMOxztipd+sZeVbeeQ38FBkuRA/P
wLtZ3JZv9C+4vxazdM/uvTuP23UPZfDnPG7DHC1m10zitnyzvt1r3MjFcsxg
NYkb8635Rge9uzR27z+yB15KY5aueRk8n8ff8Z/w3/DOvXtzmvs/CtR6KQ==

           "]]}]}, {
        EdgeForm[], 
        RGBColor[0.3476008336668591, 0.4258661076784489, 0.6147330562469583], 
        
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxFmgmUVdWVhi9FVb1X46s3vwITBkdmY0QREES7UEAREHEWClTACceoEaJB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           "]], 
          PolygonBox[CompressedData["
1:eJwtmHm01VUVx8+DN4/c+7v3d+9jEAlTKIEgBFRksIWisJAhDZPxAY95HhVI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           "]]}]}, {
        EdgeForm[], 
        RGBColor[
        0.6548091315169849, 0.5338306851976624, 0.43413776294209194`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNnHnc11P6/z/3vnTf92df7rKTJLJrpZJWyZKQsqtEVNahRfMdSsLYWpAM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           "]], 
          PolygonBox[CompressedData["
1:eJwtmnmgj9XWxx+HM/8OfvOj24RkCKUImSLzPFVCiRxT6qAokXobr+nGzVip
NN3IkDJlapLmzI4bmakQpdHY+/m+3/ePfc5aew177f3svdewf+X7FXW9NyMI
gsb8KcX/87EgaJIbBHfQPk8EwSJaC+CWtMXAn9AeKhEEI7KDYAPwpekgWJoV
BPdBvwz4kzAI1hUEwU30FcSDYEgqCHZAuxLdg4HfTgZBm7wgeDo/CL4Fbw3P
4swgGFkyCFoBr6eNQ3chMpvQPzsaBLtyguAn+nLAd0CfCTwUenHCTfCPjLcP
3qdow4G7McZqxuqPPd2w52bancjWhl4Z+V3I9ATeRt8M8LPgI7HnXmT2aq7M
8R7gO5Fvj2wH2m3wNi4XBLtLB0EamSsZeyZ9k7Ktcxbwc8gcZ/zezOkJ1rIi
8iWQbUtfHPo+ZEai/z3wY+CL4G+V5zW5Cv4t2FSetTgG/X7olZHPQj6HdlHc
30DfYjf4Pmjvwn+E9XkX+Rj2r4DeCXpnjQF8Fp2NgRvD3wX+FvQ1wbb6tObA
k5EvRr4oEgTT9f1obwAXwH8Z45VivuWY7xF0VEb/oahtGwd+DPmatJXIrwav
ALwAnhZ82wTylZAvzfwyMrzGZYBP0Pc+Y49hjvvRlYX+8uivz3o/CP4orRD9
xdjwF7aswv6sDH/zYmSbwH8A/sfQsR/aXMarx3h/wR+F/g4yE4D/ZDMvAb6I
OZ8Gb4XOstBfQf8UdL2AvQXYcxZ6Nvpq0zcH+lJaS+BL4H8XeAv6W+p70rcV
+GXGrJDrPfoJ9K+RX89cN9CWay+wBmu1H5jPROCttEHwz0BHE+j3wH8HvH1o
A8HvBu8J3Js2ALw7eDzXZ+pm8PPM4VvsH8H5uQPbz4LvBM+G/jr2fIn+PvA3
Ycw20LvQd5y9sBV6B+CezPEo9IH0dcH2SfRVYr3OwB9B9ggy64D7wl+B8erD
8x1rNwJ8bdJnUGfxc/AP4T0FvQH4Suxdh61vQG8IXhr6n9DG0DcM2k/go4H/
w3hNGa8RNl8HfQg6Pme/fAjPp9Afpu8i5L+mNWO8B9GX0H2DzE3wNgL/Ksff
bJPWC55z0O9jPv2QbYOO2uiqQ2sB/dak9+qd0FtCH8R61WXs+rRrwE+nvbey
sO934GnIXILunfD3xtYi8APQBqPjAXSvYfzuwNOwZzj27FUfY8Ea7AEeAHAB
2ZvQ0R84h75D6D9CO8N4IWPsQPZGZLqjeyGtA7zdtQeAtyc81mDwe8FHRL13
vseGxgnfGbo72tNWg7dhftvgX8+Yx9A/hTGrov9LxvsefC/yP8C7FHwLY3+E
zG2629A/Cl3xlO+KDzg/49FVXmsG/TPom7H9CWTOQ0vQeqFvT9S2PAzPUXSV
gOc7dO+l/QG9OfJzoA+B3lTnR/w53gMPwbsWmZuhnZcN0HYyZiHjt2aOxcCX
I3NLrvdsZ2wrh8yfEZ/ZmO5T+moxv9+w52P4D6BvFPwd0d8T+gV0Hmc9J9F3
DloH8LU5PhM6GxORuRTdM2izsXcl9FVZvoP3Qm9P3+/s9wzwT+G9N/TZ/Cft
UfR3Zj2y0X9S5wd4JDJl4X0a+xZqrdBXoZRtyNbZ13nKso37wdfBMxveevCs
lS7aZOj9oecAP8L80sxvP+P9KN9HXxa0Esy3G7Z1QV9ndPeGZy28m8B/YX4r
0Lkd3lOMMQ3dCXh+Bd4Ez9h8++Bl6NtDmwttLjpjzK8KOj+D/xfaQvBLmO9Z
dFViP18MPI52H7L300Yiezn43xH7sAR4C61fjs+ozmol9G1A18+0QvAG2PQ3
4+fCvx9bRtEXzfWeaoDsz9A3o+938Oehv4L8IWQf5Y6fC/x+6LulLjavBv4a
+X6KLbRnkb+fvoHQB9HuwZ6J6KiY7zutNLxT0D8I/UNoQ9G3KO1vNZzxFgBX
pR3Nt497S/4Dmbrov1Y+AX1n0J+JP9hD30bdR+AHs3wnDIP+Dfx3QcvEnpnY
c53ukyzf8bX1rZGpAe0KxiiAdzlj/CfTeygTW5cgc1eGZbpAL6Ll5XqPXw3t
s9B3Yy/FV8C5Ccc+k+FZxPifoCMj03d0PXT3DR2bNOU8jUV+An13QS/UGgC/
jvwNspfxfgY/jo0boVXke/eAv3rMe78ObQ68r8ftSxQzKHZoy3y25zqGa5f0
N9W3nYq9B1K+Q3SXjKT9M2kfI18jH3WGsZ5HnwLMAfR9Bn0lMrci+w/sWQH8
Dn3t8uzz5fs7x8zbGfmxyI8SXbEUfYeTvtN1t2sPnYTeNXRssxj5jsBH4Hky
z2dUZ/U98K7gY2i7ND5tdyn3rVKshn3NkO1HPLKL8TuivyO6+9NXEVon8E7g
g7D5I+z9WPFktn3AeuB7oD+MrrG03ejbQXsAeBRtW9Jjamx9k/GyHXwoeHXt
H+Dnop6bbJbtd8AzXPcjbTF4dcaM8G2/AK8B/LAcD7YswIbv4/YB8gW/YWMa
e06Afw3+Fa1mymdeZ18+Xr7+vMbElrnQ/wZeAP9Leb4j9sYdEyk22scZnwv/
YnS0Ybz7GPa+hGNmxc4laAPAK+rOy/WdWyvhGFCxoO78dQn7dPn2fyhm0llP
2lfqmz4Y8xnXWV/Beq4FvpC0bfIh8iUP8I3r8y0O882awF8KG3dAP6EcBd56
2Ldd8TN4BrRe9F2T5b760ErSd32WZW4AX5N0bqA9oL1QW/sZW7bQVwf4d/pm
AR9EXzfGX0/fYOY/iPnfqPNJew/6JNpPSfsU+ZYL8A8FXxN6LoqJFBvVJl5d
y33yIvKT4f039IkFvrOnAt8LfVoZ9is6nkV+Gn1ToJeUz2KsZxX/Z7tvOvD0
qHOTSeBTwOfRXgN+g/ZWaB8vX687XHf5UtqiAsfojZF9M7SvkMz80D5Ovq4X
baj8f9y6ZeMz4Mtpg1i/t8FXhI5hFMsoJ2vL/HqBP875zmAObZnLrYzBJwtu
Bu8Pfjt4EXgv8IHl7MPly48z/xasR2Y576Ue0AcAF5Yz3Iv9fDv0V9F/Nfoj
9HWUPPho7F/OGsxCdyv6zrC+E+F/m4E7J3w2xNNJvgP+DdhfCvl28DZKOned
rpwo6TXT2umb69srplBsoZxPuZ9iesX2ugN1FyoHUC7wMX1fQf+c9lGBc0Dl
gtvBH1SuQF8x8Hv6BgWOmRU7y8fL1++jHYb2En3bpJ+2E3xHaFn5PPm+Otj8
QWnHGFNi9gnyDRrzC+CxtOEFjuEUy30Z2jbF2Iq1Hwnty4ezhsuijjEVa8qH
ypcqRlesLh0Ph45JFZsqZ1buXBj33lCOolxFOb1y+zvBi0LnLMpdlHOuVC6O
vUVlHKNv0X0b2rdqDVZpL8UtqxxIudA3oWMP5VAbFXtgY5US1iFd1WK+C5Uj
K1dWjqZcbbz8U8I5tHLpt9BxNG6fK9+rmECxwWTalcAbWY/70R1Fx4vIl2U9
yiq/h346zzHYOODyadcylqHzcuXD8FTMNk9f3b9p+ybFsIplW9PXKtsxXQPG
bwPeNtt39CDgn3Xfg2eADwW/O2VYMfmHCccYijVqofMudF/HGpRlLXqg71p9
u7TvvlbQRwH3pV2jfAYdVVmbGGvUorTPyDzm9y3tArT52JfDWMOx6RC6AmQa
Irs7algx2g3yn7QQfDdnskTMOaNyx2PIdwLumDL8fzUGZLuC74R+gr7uwN1S
hvczh65R56TKTZsoxlLuFzWsO0h3URPw/pjQgL7XUs5plds2A5+Xcs6n3O84
+voA54WOhdvRlw/8Amv2To5jzueBX2Z+taBfzvxfAn4zZV3KOZV7KsZRrHMN
+Gzwa5ljQ+bbgHY3c59KX7V851DKpZbQ2ufbp3+X9B2mu+wQ9gyX/4i5tqMa
jWo13Wm3w39KObbi06Rj+3+pJpZ0zK7YXTFPR+i3RJ07asx1AH2wIYP9Xhf8
TuDWUe/lLuDLoL9A32P55nkeeFbKc1EOqFywX9p750/wGtj2GPK3lvAch0Jb
As+1EcdY7yadgykX05nQ2SiKuvalGp9qfUfpu1hrj/0/Aj8ZtW9TTjJUtQZ0
nMx1zlsIfVzUsZJiTMWauaFz3wuKTxP2YfJlipkVO8/CpvtFVz1Q35f1rcn+
7YHNTyjfB/8WvJg13sF8csGrg1/G962i8x16beSzI6FjYsXGypEWx50DKhdU
jr0AfJ5yhFzXdOYrvk/47tgD/mfUNSzVslSzORh1jqtcVzm0cmnlHMo9TrB/
TzHejKRjN50xnbX2igcC1wyXp3wGdBY+ZY8OAP4t7b1+jvX4VflH6FpeZ/SH
2NMImV15PtM628Oijq1Vc1DtYY/usHzXSMaBj4b+Sr6/0aPg30GfAD6etlv1
1NC5+Avob4b+aug8lu+cR7mPamyqtdVhvIHgV7PG9SKukahWUjNm2nWKgcH/
i86ntFdYn43oy427VnJS+Xnob6hvqZxTuecxWmG2bTwOXAIda/Odc/dANgHe
iG+5hr4kcHN4liK/H50tgOsmfVe/Cv/1wGd0Z2LvEvjPqv6Yci3mVugHZVva
c1eN8UngcsrH8l1zUe3lTdaod4b7LgLflXLtWTG1YusRCecu8jnDVa9jPldw
tqdz5ovYe6Npp8s4xlCskSWfk+099ZtyReZUL8sx6nJot8TtO1QDqMR4J5KO
FReUxRcjfzVjfpPnmsNt0F5J+S7XGmutt9H+J99nfjtw36TvFuUMyh0GJn3W
9c317U9jQ0lsq8sa1sX2tjHn4vJZ8l3KAZQLaMyTyg3iznXWMudvgD/Gxlf5
lq/T5kP/JeZcUTXsw+j7LuravHyIfElh6Fq6Yqo7wE+lnZtUYfxfgBsyXvMs
1xgaAN8Yd+1TOXRzZMegvxtj76O9qtwOnuI8x4xtgTsknHsoB1IuNBC8ZK5r
ooXKVVK+K1XzfRm4TOjYXWewNHBX1jhdxjXbLYzdLuZah2pUG5KuEatWrBqT
ak17k87d9KaxR7Wi0LVi1UQuUy0q4b1flTYb2RcTzpXPYH911YLgORbxm4ne
TsanXXuuBn0C8D7V4CLO+ZX7K4dULqmc4mPgX0PfLdpTvyj2S7t2pzeRzVqU
tM+KztCzwDMVn0R8h46VL0q4lttAMTbwltC1PJ1BncVk3LVD3TFXqDabsi9T
DlsEHIbOHfXNUsDbk85d5TPkO4akXTtVTqzc+CnlsJmOaRTb7AY/kOkcWbly
w7h9n2IgxUKPJJz7qSYyK+mcTbmbfIh8yQ26n7McQ7WCfy/jfRjxmn2l9xDa
1kznXMq96kI/x1qc1x4Azo15L33KmlcGvp6+M9DeRr4O8P7QbyWK+Uajf0zS
e1978CHgFci8CbybPZetfDF0bfM8+uoknHMo91DN5bW4cxzlOjrzvcFXJfzW
ohqCagnjoB/krNfge9yguxZ660zfcf2ijvkV+3eE/wf4xybsK9rDM1rfM+nY
RzmsctnclGM52ZiVcg6jXEY5V1/kx5Tz3dIT+uPIdwe/GHonxtsJ/b+hcwvl
JMpNxCNevcEcV66fci1ONfA5uq8T9o3F6P9C/pE1/CbimtFRZDcnXXtXjUG1
hupxvxUclg9B/9spx2qKoRRLnZMPifgN41f45+t+yPcb0VvAC+Wz8l1jWAT8
RcJvj6opqLYwTmc2zzUK1SrioX2P7gzdHTel7DsO0uppvjHHsnqj0VvNOdVj
Ml3zUO1jbNK5tnSeBH4f++ZEnIMpF5sT89vScu05aC1T1q0agmoJm9LeK6rp
LYP3lpRrw4qJFRs/EnVtW2M2xf4JCeuaR1sZ852hu0N7RHule9K1BtVAuyVd
E1NtTHdKlLkuC517q4amWtpPaefO+iYngL/Xe0rEZ3BXzG8AegtQTXEYY11I
O/bQN/g77TcPvX3s0B0I/EfasGqgqoUeSftsyQceBk5gY8fSrqErcP1N6xvx
N9S3HBm6FlvMnOdgyzNpv/UoJh8cc81ItaM99GXA+3vMsN4Q9JaQSTsQ8Zut
3m7/gn4w4j2hvfG5ai7MfQN9hxQfJV171Z1RlPQZ1VmVTYHe+mLei3qD0VuM
YlDFotKZDT4/4bcsxQTzVPtlfl+U9huj3hq157X3e9JXDtrAmGNPxdSKrd+J
+21QMapiVfGIV3fiQnhvi9oX6s1Ub6fFMceyejPW23HTlGtVqqlU1n6NO9bR
mdXZ/XfSsYdqQqoN7Yu5Nq43T719blb9JbCPk697POrYWTGyYuVSod/itKZZ
wFXjzo0UYyvW7hT3269yRuWOz8RcS1GNQbWGf4HfmOmalWpXq2N+q9Uaaa2W
R11L0J7Q3mgYdeyhmrlq53oT19v4KfbEH+DNoq6d6Y1Vb63Pxe1LFbModknE
zDuVNgO8f8y5ud6E9Ta8LeZvrxj2Kr7/3eCDI67xq9avGrdq3Yv1BszapuK+
W0vq/Vn20EoVOIZWLK07UXdjNn1VoKXj5hWPeKvR8gr85qi3x0uh5xc4BlQs
qDdtvW0rZh6c9hu53sqX6Q4BvyTu3F0y1UPHqIpVpfMq8B9Szq1Kg9cELxe3
LXrj11t/S9UrC/ymqbfNHgn/tkFnQGehRejYvhb0Vor9GPODfH+zDN1XoWl6
c9HbS/nQv73QG04F1WvS/x/b01eYdg6iXERjtgtdQ1ctvZFylNA1O9Xu9IZc
C9tnhv5WqkmqNlkndC6tGl9t4Bqh56Y3Pb3tdQqtS78x0G8NVNNTbU8xrmLd
tqHfbrWntbcVMyl26gPPJOCXaA9FfMfprtNvFPRbBc15EviL4H0yzfNy2jVW
1Vq1R6am/QaktyD95mEY8Oy03zYVQz0HPDHtsVSzV+3+mtBvKarB1AJerff2
wG94a9KOkRUrKybZCjyP9lrEMbRi6Q/SfuuTT/kw7ZxFuYtynsWKB6KO1SSz
Hvr0mGuX+s3CYvB1Me8lxTiKdU6nvLeVAz2t3DLttyzFSAfSfgPUW6Bins1R
xzyKfVTT6aa1i/ptX2v0JLr/F+c9DaU=
           "]]}]}, {
        EdgeForm[], 
        RGBColor[0.90990693759858, 0.6447673439964501, 0.32071150655559805`], 
        
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxFnXW4VdXWxvc5h8PJ3XlsBewWri3YiVKCga2ohJikn6KYgN2dWAgKCIp5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           "]], 
          PolygonBox[CompressedData["
1:eJwtmnfgjmUXxx/jtwfP/Uyln+wiGaFkjyKS7BRtka1lRig7kVGo3qISskMI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           "]]}]}, {
        EdgeForm[], 
        RGBColor[
        0.9589817455618589, 0.7674543639046473, 0.47284341124176266`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNnAeYVdX19u/M3Okz3Ln9DhakqICAIIo9MZgYgzWJBCRRFEvUKBY0QQkd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           "]], 
          PolygonBox[CompressedData["
1:eJwlmXm8jtUWxx+c8T3nlHd8nqOBE6WrOI5rKEOJUhmaOZzSoJR0c5FuUd2i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           "]]}]}, {
        EdgeForm[], 
        RGBColor[1., 0.95, 0.75], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNmGuMVdUVx89w78zceXHnvubeyyMWPmBMKVrbflBpRECJsSTSLxRoFQbT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           "]], 
          PolygonBox[CompressedData["
1:eJwtlUtsVHUUxg/S6WtmLO3cmTstNEYTYxqLC4wLTQiIVBfqAlcUN1iNQRdo
BBcCWou0tAmBhVBDQUCRotSFMWiQaGKVmJZWIIaI+EqUoDVQrNgivoi/L5+L
c3PO/7yf98aOpx5afV1ErAMqgKuFiBNVEc01EV1JxDLoocqIOxB6uz5iFZDJ
RnTURTzRGNFSjHgf2arqiDkNERfgn0Z/P28JvJXILEQ2D78Re5XIzMAfgJ+F
f6EccT4fMQ1/Ct0e/E3DzwK94BNpxHgu4nfkJ9BtRmcIfBZvC+G1Y/Myuhux
0YOtd9BZhu5R6I+hO4rWXUvMY6WIudBvQY8h8wv+Xoe+qzZiCW9D2OpH/yZ4
M+S8Afwl6aDbDMwUnINyOYJ8a9E5KtchdM7DzxDTJ8hWIlNE9m5irq5xzlno
Ruwdmk3N8NkEXiq5toehU/C6BuuqpqptK/Ymsfcm+b4LLylZthP6ALx7gJOZ
iOPUeA21niDnc8odf78Rz828bbg+4j1kahPLSPYnmr0Ye1XI5IhnGpml8K5B
f1HlGv8DfhuwvtI9VW8nsdGp2cDHC/D+AA7VeGb+BC/DnxvuuXq/lPhqco5R
sSomxXYL/OXwR6A3VnmmjpPbSvTPEM9+avAI+P2JY/16VsQe/O+mJin5DyCz
C/xT3o5hvwVYkVpGsjuJqRZ+OznOwfcI9rrg/1rv3iqHi9h+DZk7az0DmoXN
Bc+eYp5Edj46l7LugXqRwd6DFa6pars+cW3vA56F14bMVezn0L9X+dYb14xr
1tvwcarGNZoP7yD0Inx/iM3BonNW7mPILC56xjXrXcj8CH0QGw3EVw9sLXhH
tavq2b/QU4lnOQfdB72oaFvXsHkD8W1Cv5NYXgSehz9c9q7kkd8DPkoPtmT+
r1nJMSm2ZmJYTiwPp671MG/tqWdGsyOdaux9ydsR+B8AI/i7UvCuqYaq5ee8
jWfcM/VubYNrpxlbR+yP4u9iuEaPgf+N/skqz5hmrSNxLtop7VYL9Ah4D3AF
vAmoq/ZN0G3YTU4v592zXeBnU+/ec8zIV+DnUs/GCWI4hb2+xLvcxk72gq8u
eZY0o6P47+etF/wH9HeAv1F2L/bhYxD8MjKfQZ/F5jbyayx6VvXWBH6Mt60Z
10C10A5pl9QT9UY7od3QzZ0q+EbqVsrnGPSKxLdSOSv37rJ7qZ25hO290Dvz
vqG6pa8gQ+mjlc+OknusXktmX9k3TbdNN3Eevqah52V983T7+updK71tgu5W
DfLeGe3O9pJt60brVi9JXUvJvKpYgAWz3YMB8L31rp1q8EDiGqlW6vFf2H6y
0f8W7cQq8G9S9+Ij7H8LPkrNjuY8Y5q1LdCPZ/x2Rr1MfSv11k18B8rujW7i
z+Dbi/5XKabT+F7T4N16Gp3+1DOr2dWObSOewdS3VjdHt2dB4t3SP+Z28O9T
x6ab/h14BTCc9T9A/wLZlG29PaN/X+JZ1T9G/5pbi/6XaGZL8DeX/S/TDdEt
GS/536Wd0G78B4iUHt4=
           "]]}]}}, {{}, 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          
          LineBox[{4640, 5977, 5979, 5978, 4547, 4813, 4548, 6423, 4772, 5196,
            4511, 5269, 5270, 5068, 6481, 5067, 5387, 4614, 6438, 5131, 4673, 
           5218, 4639, 5439, 5440, 5217, 5216, 5266, 4590, 6277, 4989, 6276, 
           4990, 5357, 5356, 4591, 6431, 4988, 5014, 5365, 5366, 5161, 5210, 
           4592, 6379, 4593, 5958, 5957, 6279, 6278, 5209, 5146, 4630, 5419, 
           5420, 5145, 6496, 5144, 4400, 5453, 4397, 5445, 5444, 5476, 5475, 
           5836, 5835, 5474, 6464, 4840, 6120, 5780, 5781, 5271, 5273}], 
          
          LineBox[{4871, 6468, 4872, 4771, 5833, 5832, 5831, 4697, 6017, 5178,
            6115, 4816, 5953, 5954, 6301, 4997, 5537, 5538, 5291, 4863, 5360, 
           5001, 5428, 5170, 5171, 5040, 6336, 6337, 6302, 6304, 6303, 4998, 
           5000, 6477, 4999, 4589, 5956, 5955, 6515, 4672, 6444, 6401, 5195, 
           4510, 5386, 5063, 5742, 5065, 6365, 5064, 5834, 4456, 6505, 6506, 
           6504, 5543, 5542, 5976, 5975}], 
          LineBox[CompressedData["
1:eJwl0TlLnGEQwPERtBFJPHbX3VULCwUh5huk8BuoTbyq9FHwQNBKo6AQFEHF
q9S1sdFPob1ojEcheDfe8YjBH6T485+ZZ5555uWt/tbV3JkXEf34UxbxivZM
xAtPlkTccqv8jrPJiAzO1Rd5CbnyiBUkUhFJlCOFNT1N7jQire+aW8y54fN0
xBlWkMMyJtRX+UOpd8WbvGPuFu/yTx4xZxSb8g35J67Ho/4L71U4q8SD/Icd
7nmEx50NJyJ+6d1DWi2LDC6dHaod4TcOsI/v+scwjlNzvtr9hBPmX7nzWVyk
NiUe0NNrnwZnfTwonxEXco2eArVaeT4v+sYlzGEB81jXt23OtJ6UuFhfnfgj
J+UF9slHt3yWe/jY+RdxFf+1yz90eOuNh9x54jb5c9n///oOdXZK1g==
           "]]}, 
         "0.1`"], Annotation[#, 0.1, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwVz8sr5XEYx/HHimLh7kjhoEQk2Sg70lhM6GAWE4fkvmCl5lhOuSxcFpPy
B4xLMztWsxpOYTU0f4CVhdTMRi5LXhbv3p/v83yf5/v7JSeXUos5EbGClqKI
b+URPWURW9hEVm0+EbGAGcxhFnulEcd8gjEzuyURab4vjmjm/9zE/7jUnTJc
OV9j2t6myog6vR/yT/Sj3VsD3MmDXOWN4YqIEXxCq1qFme+c1quUO+QET6DN
znG+waMdSee/8pNc614SYW7NN2xgHRd6t2qXvKO/jSn/8lEvpT6EIzvGvF/P
o9zAaf7Ard7o41w0ms3jLDrVz7jInkJk9Fac77jA/gZ7e93Jl6vVavAq16o9
+JZVM+P4rX76Puu9DLrt+MLL7n2Wv8p/3DuUE2rnZq/kLvmaD/Bsfp9f+Be/
AXNCQYc=
           "]], 
          LineBox[CompressedData["
1:eJwV0jtI1XEYxvFXUiiaPF6yNi1TcMu75qJDwekYKBxNU0PByQsqFI7S0OCm
riWVoIsNqYkGNTTWmmmjouWiLVmBiZ8zfHne5/m/l9/wL+4fbRvJiogZfM2N
2M+P2KLjRRET6CiMGMqL+CE/wDffftJDlCONDVlhQcQm7eB79N+8ElGGj7ID
ez7QY/oLR8jRd8PMdZz6ltI7r35qttPN5+oX+KLvIr2EUvXdqxH1ekr4bP4C
mmWniYhF+ZndSfsG0M6381W0iVbTNf2r+I+3WEGlvNH3W/Q2vWP2ERrsHPOu
RjpOp2WX3Rrhl9UJWYWZGjMn3vwH7/mWzJtQ542t/B7S5nbpjuw73qHGnjVa
S9fpb/OrtJoftrsqc9/cEzzGIJ+0I1e95P62GztIyVLecI9OyWdlc9jk7+vd
oJ/4v76V8FP8P/Ub+67xn9UL7j3jX9LXeIV+Ox+iB93oQ2/mnzD/gHYhjc5M
hgFMunMO6h9VEg==
           "]], 
          
          LineBox[{4569, 6428, 4800, 5264, 5263, 5663, 4826, 5662, 5661, 6016,
            4696, 5536, 5967, 5635, 4788, 6095, 5634, 5899, 5898, 4507, 5262, 
           5496, 5495}], 
          
          LineBox[{4637, 5544, 4638, 5545, 5928, 5665, 4836, 5664, 4835, 6119,
            4437, 5172, 4838, 5666, 4837, 5667, 5230, 5448, 5447, 5446, 6122, 
           6121, 5670, 5462, 4675, 5569, 5669, 5668, 6403, 6402, 6467, 5824, 
           5963, 5962, 4595, 5358, 5961, 5960, 5959, 4991, 6283, 6284, 6280, 
           6282, 6281, 4403, 5790, 4398, 5783, 5782, 4641, 4831, 4830, 4530, 
           6419, 4754, 4874, 5010, 4636, 6440, 4834, 5220, 5219}], 
          LineBox[CompressedData["
1:eJwl1Hloz3Ecx/GP6w9lZTe2f5iiFduUzeYYkqMWJRK1fyzFxOTImHIMOWdD
CP8g5xybOcJM/jDsIFfJpsz2D5Fc/yDz+OaP516f1/v8fL6t3+CFJbOX9wgh
xPuTnhDCUhRjNcqSQngXH8J62kE30BP0JP4MCGFKbAip6gbFhXA6MYQt4nvF
K1CQHMIv+S/yv+kuuRPiu6N+2oN+ldtDR+vNds7BXv6U/Hy7FuCHXOnAEA7Z
UWfmYVqDbLErdKhYGobgCH+Npkez9e3gN0f3Qjke8Ln6GmkMhov1own2PqTb
6VFaixR1Le79yrmVnjP3PM5it9hLd7xnzwvaQD/Qj9iEzehj9l3xLDoKvdEL
N81qMqMZj9GKFhw38zo9Rm/QSnUH6Ts9HWhCC5pR5LvsxC70dOfP9n1Cpn25
NIPm0f1mlMsfoDHecz+6D8rlSukWupbOVTMHK/hifqS9GXiir0t8mT3f7P2O
7UjhU9HkrnnmNtM4pMvF0kI99fKL9M9CkZpCudmoQiWeij9T12ZnO96g0e5t
YltRhfV8gd4yOgGT1Yynz+U+6b/EX8ZF1OAKquR/yCe6xzC9yc4/xUc6zxNb
TWd6W7f+NOfXYll8Juqcn9Au9Z14q3cin48JWGf2JPGJuMBXIwWpqDGvVjzb
u3PQ6Y1rzVtD+8o30PH8KudxdKXdk9VlR2/Tl0Pz6V+5W+K3cQdnxDPdIyPC
jEw8iv5fUWFnhZp9mMFPx9joO+tpd062Kwkj+DHRbHXdqOenyU9Frvo8tDm/
wQUz4/lqGqu3f/Q91L/XP9fdNvIlWIGrZtVhqtr9/GL9SxL+/478A4hmop0=

           "]]}, "0"], Annotation[#, 0, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwV0TFIlVEUwPHjkmAlZOrLxSgwKLeWEjTCrRZBEwQdLGjqpUPUUPqeEBpk
Uy5Gpa2VQg06JETpM9JRJYfqWZFWpOYW5NLvDn/+95x7zrn3u9+RS72tPSUR
MYGTByLqKyMe4SHqKiIu8zkuVEWc53m+WB3RjRzy+HUwYv1QxE+usl/NdeIM
H+NiqjWnryai34xmNTnOy41aP8B+62F15XyP93EXOhPu1YBCJuKpvjl+xsto
NHOJ39vfMueqeJvz/Jtv8W37m9YF3nanr2oz4m88I3dKzStetddm/YFXnfvF
fq34mrhDvCD+7uwVtfV8AsfxPL0LH1a3wq/TTD6NBmxgHcv6e8277+weHuE+
/EF/ehNM6F0zp91+kSvkNvQdtS6Vu2k9qGbMvCcYxw+5j3Kf0IJZ7zeHAs7a
u+PuzbzrXzw2Z1f8D3/lSniRa8ye1DuQ3g5D1jucNeMK3qqbxQ39OXOuc56n
0gz701xpRlgPp3+qP8uD8ll9Ofdsk1uQu8BlevfihfqX+Jz+XfoGXkMRd/XO
632HN85rkjuDUuzBfysTa+A=
           "]], 
          
          LineBox[{4500, 5894, 4726, 5177, 4450, 4426, 5700, 5701, 5699, 4920,
            5702, 4867, 5690, 4868, 5470, 5471, 5733, 5734, 5732, 5032, 5604, 
           4728, 5494, 5605, 6414, 4506, 5606, 5237, 6061, 6062, 6060, 4753, 
           6113, 6114, 6111, 4812, 6112, 4529, 5201, 4971, 5938, 5939, 5940, 
           5331, 4927, 6211, 4928, 6212, 4568, 6264, 4976, 6262, 4975, 6263, 
           5332, 5333, 4974, 6260, 4973, 6261, 4509}], 
          LineBox[CompressedData["
1:eJwV0mloz3EcwPHvRtmEnDPKAzmmIZR7o7BhtGHuHA/cc+y/IQ+QFM+YnDGs
nLk3HsiRkWMK0RDlXJjIMBNyltfvwXvv7+fz/Rzf//7/9rNiuflxIYR0fyqb
hfCodQj3uS/3wXjkIq55CB3bhPCE96GiRQhjxTe4smUIV/UkiWeLj4uP4Sja
6m2DZExwN0quWG0Cz1CfyKXyZQjice7Kk0LI4fN8Ds2SQ2iKf/beUn8bHdAL
2erucBb/U3PYnD/qxjnnIgdjMQZFanZ4x0aux/EoVLsJve0u4i1yi3gzL+YC
9HQX40x7MjDGjuZyx8w5Ij4ozhEf4pveW4H+6vPNGMDlrbxN7i+6qennLb/k
E/g3J3J/cxrwT/EpdaUowzL5ePnv8nH8g+txH/nA38TH1Z3ASXQyP8/eHvIL
uCcv5Bzu4C7BO9d492H9X+SGiOv4rdxu71zvXKK+oVmXnB/IL9dXzTV63nCt
eKPzBhRhqdzgyPLtzHsp/iCu4gdmPYx+Uyg3b6SaEXjs/oX7Qud0d8+dB3Ia
Xtn9GgVyl9QVcrX4Dbr6zKkYJF+pt6l9B9w39vb9nC0/054ycSqXclc+zU/1
P4u+A3SPZuGrfXXYra+LeKoZ9c1McZ6sZ7jeWvdfsEduL0qwze5pvJ2nc2L0
m/CuFhgtHoWluBLNMXOOmYXOE80s4BjO6Lmu/gauoQovcVt+kp7LarZiCzbj
XfR79M4YekTfl9x18zLlr3EGf/LOz6jBB/FHnqK2WO0uZKmrkW9tTzKS0CT6
DtylRb8h912i3yCP5ps8zH2W92fwEIzUM5QzeQSGY4U3pdmz3r5p+tbxdC6O
/rfYidXiXZynrtKus2ZU6G3MMV7AS3g+5/NaXsxzeRHP4/P2nMMFXMRHuWfm
NfLOu873ov+pPavEK/Eeq8QHOZFTkOqzdOb/Q5q2Ug==
           "]], 
          LineBox[CompressedData["
1:eJwV0b9LlWEUB/AjmJMaeb3+GCwd2gK1a6uNOthQaaiTRSIE2WwUNIjpNVAL
NwtsiFTQtNT3ejXxf9CWoLSxNsXUWvzc4cP3ec953/Ocqw0PntwZLIqIGZor
IgYqI/plZ01EVyqirjriMlf4odcg6+nSv6d/1XnrUsSB3i961fr0euR9+Uyv
STbTSIbr1Olfk09rzXcekm18Mu+5nLbDkvMKy9zyvFMVMeac5RXjfEtH7LLH
Mp9Z4YV7T/XPOOaEv8zaccZ9ibs3yHFo9le5TZ4tNvng3RO9YucSLjBibt78
TRI2yPGdMe9m7J6Vjzi17xkvfbPu7lH5z/N/lpxTaouyUqYJ3lBEtx3TZiX6
P+2xzx+134W/q1o7q+Z8ocxepYX/hztbPN8gw0W1ch56t4M1tdv2HOQx02qT
+sNmTsjXTJEy59B9R0zqfVSbZ44Fjn1717edhaTV+++ZJa++L2/a/UC+I/F7
3sqc3JZZcxN5DiFUYVE=
           "]]}, 
         RowBox[{"-", "0.1`"}]], Annotation[#, -0.1, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwNzr0ug1EcBvA/d6BVH5suLMWqJo3YfLSGpkIi6UoIFqXF5iPpxNboTaAu
gbU1cAm+wg0ofsMvz/m/55znvOny9vJWT0QUme6LOBqOyPRHNLkmk4h4SEXM
y5JcYXIg4ts8Ib9k27ey3JAdNnXs6Xqy3tGxy6X9U/MZ5yTdTXDj3KJcYoEC
eUaSEeND3uDV/MY7J7oq7nxaf9Cx3ybtfFXWqHDIAbfO/nrvjy4/XHDlf3K6
ju3XByOefXuhZl0lb//R3NB7p+eeFqPmGe+OyZzslVMyy6rza6zrnGVf/5z8
B/G7NAI=
           "]], 
          
          LineBox[{4436, 4455, 5473, 5472, 5527, 5526, 4570, 5640, 5641, 5638,
            4791, 5639, 4508, 6096, 4789, 5636, 4790, 5637, 5968, 5969, 5717, 
           5718, 5714, 5716, 5715, 4434, 4454, 5244, 4729}], 
          LineBox[CompressedData["
1:eJwNzrkyhGEQRuEWGFKMLZRwBRJraAtMSSnLlC2yL5FroAozMrEtQQkYjBEj
YrgCsaVEtic4dbq//+3+uyE9OzBTEhGTWEtGFGojLpDDOa5wiaiMyPMNrrGr
f66LeMEjinjCnR35ioh39QfekPD2yXtVEfv4U6dkTtUnOEZ7vXk7m2SL3KFv
VH/JJvUZzmIT29hCxo57mSzPVUfMosWurpqIA+7mVp73XubmcjzIL+kXsYA2
33vkpjGFNbx6S8n1o02moL/Fjv+su6XZbRv8oy+T+eVvPpQ5wgjG3TfKEzzM
afSZG+Oc/CD36oe4FGfmE7zqxk67l3gFy/gH2rlBBA==
           "]], 
          LineBox[CompressedData["
1:eJwl07lPVFEUx/FjoYkWDgOMMLiCJkqIJCJioxILTQQbJ1ZEXBAYQSMWkGjs
3Dq1dI2MilujjbExWhlxH8ANEkn0b8BExfXzYvHN995zz/nd995kqjv6cgdn
RMQwHpRFvK6MKOINRrAmHbGAF2I+FmMRSvRWcSlnOc1X50UUMIjdFRG7cBSX
cBkf1N/jHdpLI7qyEd2Ymbg8os1djfgu77y8c3hu/QJP8QzD6NXzkl/hrawx
jCKlfww5eaPc4nwTb+HN3MqVXMtVvJyzvILHebu5iaQfTZmItWj2XP3uOy1/
gB/ZP0YN6rFB7b75jXxTzw389K7T2OYd22SsUsvLbtfTYL1MfmPybbFfX4O+
A3zMed76SvJMmG3muNoT95zgOfa16oMoYJ/eM+Z6+Cyv5lvyb+MOdpr5rT6l
/ov/4g9y9lP8Fdf1DaFZ7w50qXWjE/X68nxYfZa7O5N86xLvlMakZzhi9ov3
+Ix7nvMu+vQMJL+z8/7kN+eU/rkYcnZS5il8cjaZfBv1Ovkr5Xeo70VRbVrm
Ov0jyW/BP+wvyrqAb+4pqKXMFe0n5FzDoeQbydkjY33yrc1knH+0HsdDc2X2
S9XLeQnXoBpN8nr09GKrO1vRkvn/v/gHVJFwdA==
           "]]}, 
         RowBox[{"-", "0.2`"}]], Annotation[#, -0.2, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          
          LineBox[{4413, 5165, 5369, 5368, 4598, 5626, 5627, 5625, 6087, 6086,
            4475, 5644, 5645, 5643, 5647, 5646, 4611, 5413, 4610}], 
          LineBox[CompressedData["
1:eJwl0UsvnHEUx/HD0m06454pNW9AypZgZSFi4QVoTBPpFCNIjI24bHgXxG1F
YseON9Bu7KoJmxJN6p6wKp8nFt/8zuV3znPyf3L5qeFiWUTMoCITsYLO5ohl
WqiNyKJf/pFu1EWs43NDxI/6iJ8YwzXGecbTEX/FE7SLb9SOb/ICimovdJIu
m1/BEv7Z+8r7H90InjKcqg/Jt/hn+fLoQS++65/TC+zx7Cdz/INNEQO0pjEi
hV2z02o7tEpejTWs4srss95XN5bMbyffwiae9B7xgCP7TtSOcYCSXZfqf3CT
3I8FtXI33YvvcItDc338c3ol+Zl8jv6m8/QX/ZS8kXdrpV+QdUu7PS3iHPJ6
bbRSPZN45LU0hTRG5B/oon6HeCnz/g/fAOgOQaM=
           "]]}, 
         RowBox[{"-", "0.3`"}]], 
        Annotation[#, -0.3, "Tooltip"]& ], {}, {}}}], {
    DisplayFunction -> Identity, AspectRatio -> 1, DisplayFunction :> 
     Identity, Frame -> True, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], Method -> {"DefaultBoundaryStyle" -> Automatic}, 
     PlotRange -> {{0, 1}, {0, 1}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
       FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           GraphicsBox[{
             
             RasterBox[{{{0.148, 0.33, 0.54}}, {{0.3476008336668591, 
              0.4258661076784489, 0.6147330562469583}}, {{0.6548091315169849, 
              0.5338306851976624, 0.43413776294209194`}}, {{0.90990693759858, 
              0.6447673439964502, 0.32071150655559816`}}, {{
              0.9589817455618589, 0.7674543639046473, 
              0.47284341124176266`}}, {{1., 0.95, 0.75}}}, {{
                Rational[-15, 2], -75}, {
                Rational[15, 2], 75}}], {Antialiasing -> False, 
              AbsoluteThickness[0.1], 
              Directive[
               Opacity[0.3], 
               GrayLevel[0]], 
              LineBox[
               NCache[{{
                  Rational[15, 2], -75}, {
                  Rational[-15, 2], -75}, {
                  Rational[-15, 2], 75}, {
                  Rational[15, 2], 75}}, {{
                 7.5, -75}, {-7.5, -75}, {-7.5, 75}, {7.5, 75}}]]}, {
              CapForm[None], {{
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], -50.}, {
                    Rational[15, 2], -50.}}, {{-7.5, -50.}, {7.5, -50.}}]]}, {
               
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], -24.999999999999996`}, {
                    Rational[
                    15, 2], -24.999999999999996`}}, {{-7.5, \
-24.999999999999996`}, {7.5, -24.999999999999996`}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 0.}, {
                    Rational[15, 2], 0.}}, {{-7.5, 0.}, {7.5, 0.}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 25.}, {
                    Rational[15, 2], 25.}}, {{-7.5, 25.}, {7.5, 25.}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 50.}, {
                    Rational[15, 2], 50.}}, {{-7.5, 50.}, {7.5, 50.}}]]}}}, {
             Antialiasing -> False, 
              StyleBox[
               LineBox[{{7.5, -75.}, {7.5, 75.}}], 
               Directive[
                AbsoluteThickness[0.2], 
                Opacity[0.3], 
                GrayLevel[0]], StripOnInput -> False], 
              StyleBox[
               StyleBox[{{
                  StyleBox[
                   LineBox[{{{7.5, -50.}, 
                    Offset[{4., 0}, {7.5, -50.}]}, {{
                    7.5, -25.000000000000004`}, 
                    Offset[{4., 0}, {7.5, -25.000000000000004`}]}, {{7.5, 0.}, 
                    Offset[{4., 0}, {7.5, 0.}]}, {{7.5, 25.}, 
                    Offset[{4., 0}, {7.5, 25.}]}, {{7.5, 50.}, 
                    Offset[{4., 0}, {7.5, 50.}]}}], 
                   Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], StripOnInput -> False]}, 
                 StyleBox[
                  StyleBox[{{
                    StyleBox[{
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "0.3`"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -50.}], {-1, 0.}, Automatic, {1, 
                    0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "0.2`"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -25.000000000000004`}], {-1, 0.}, 
                    Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "0.1`"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 0.}], {-1, 0.}, Automatic, {1, 0}], 
                    InsetBox[
                    FormBox["0", TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 25.}], {-1, 0.}, Automatic, {1, 
                    0}], 
                    InsetBox[
                    FormBox["0.1`", TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 50.}], {-1, 0.}, Automatic, {1, 
                    0}]}, 
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], {
                    Directive[
                    Opacity[1]]}, StripOnInput -> False]}, {}}, {
                    Directive[
                    Opacity[1]]}, StripOnInput -> False], "GraphicsLabel", 
                  StripOnInput -> False]}, "GraphicsTicks", StripOnInput -> 
                False], 
               Directive[
                AbsoluteThickness[0.2], 
                Opacity[0.3], 
                GrayLevel[0]], StripOnInput -> False]}}, PlotRangePadding -> 
            Scaled[0.02], PlotRange -> All, Frame -> True, 
            FrameTicks -> {{False, False}, {True, False}}, FrameStyle -> 
            Opacity[0], FrameTicksStyle -> Opacity[0], 
            ImageSize -> {Automatic, 150}, BaseStyle -> {}], Alignment -> 
           Left, AppearanceElements -> None, ImageMargins -> {{5, 5}, {5, 5}},
            ImageSizeAction -> "ResizeToFit"], LineIndent -> 0, StripOnInput -> 
          False], {FontFamily -> "Arial"}, Background -> Automatic, 
         StripOnInput -> False], TraditionalForm]}, "BarLegend", 
      DisplayFunction -> (#& ), 
      InterpretationFunction :> (RowBox[{"BarLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{
                 RowBox[{"Blend", "[", 
                   RowBox[{"\"M10DefaultDensityGradient\"", ",", "#1"}], 
                   "]"}], "&"}], ",", 
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "0.3249005919687481`"}], ",", 
                   "0.18452575196532695`"}], "}"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "0.3`"}], ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], "\[InvisibleSpace]"}], 
                    "GrayLevel[0]"], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "0.2`"}], ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], "\[InvisibleSpace]"}], 
                    "GrayLevel[0]"], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "0.1`"}], ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], "\[InvisibleSpace]"}], 
                    "GrayLevel[0]"], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"0.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], "\[InvisibleSpace]"}], 
                    "GrayLevel[0]"], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"0.1`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], "\[InvisibleSpace]"}], 
                    "GrayLevel[0]"], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}]}],
              "}"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
           RowBox[{"Charting`TickAnnotations", "\[Rule]", "None"}], ",", 
           RowBox[{"Charting`TickSide", "\[Rule]", "Right"}], ",", 
           RowBox[{"ColorFunctionScaling", "\[Rule]", "True"}]}], "]"}]& )], 
     TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.6155656295179806`*^9, 3.615565638828513*^9}, {
   3.6155658020638494`*^9, 3.6155658163286653`*^9}, 3.615566771757313*^9, {
   3.6155670835491457`*^9, 3.6155672268873444`*^9}, 3.615567257129074*^9, {
   3.6155691675533447`*^9, 3.6155692201173506`*^9}, 3.6155695129841022`*^9, 
   3.6156443374968076`*^9, 3.615644686952795*^9, 3.6156458127781887`*^9, 
   3.615649051410428*^9, {3.615649772871693*^9, 3.6156497871515102`*^9}, 
   3.6161623470971994`*^9, 3.6162596167162313`*^9, 3.616260311359963*^9, 
   3.617554128869655*^9, 3.617558269160466*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tutorial 2: Improving Accuracy", "Section",
 CellChangeTimes->{{3.6156445630717096`*^9, 3.6156445695870824`*^9}, 
   3.6156555592962666`*^9, {3.6156556232879267`*^9, 3.6156556274631653`*^9}}],

Cell["\<\
The first step in creating more accurate persistence calculations is \
increasing the number of thresholds and shrinking the spacing in between them. 
Here we define the settings used when calling the Verified_Filtration \
function, and calculate what the top threshold is.\
\>", "Text",
 CellChangeTimes->{{3.616260164392557*^9, 3.6162602409249344`*^9}, {
  3.6162603259507976`*^9, 3.6162603533833666`*^9}, {3.616260621851722*^9, 
  3.6162606255529337`*^9}, {3.6175593034486237`*^9, 3.617559304985712*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"StepSize", "=", ".01"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NumberOfSteps", "=", "61"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"BottomThreshold", "=", 
   RowBox[{"-", ".4"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TopThreshold", " ", "=", 
   RowBox[{"BottomThreshold", "+", 
    RowBox[{"StepSize", " ", 
     RowBox[{"(", 
      RowBox[{"NumberOfSteps", "-", "1"}], ")"}]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6156558584673777`*^9, 3.615655885550927*^9}, 
   3.616260318571376*^9}],

Cell["\<\
We can manipulate the contour plot of our function so that the super-level \
and sub-level sets of only a single threshold are highlighted. 
The light region corresponds to the super-level set and the dark region \
corresponds to the sub-level set.\
\>", "Text",
 CellChangeTimes->{{3.6157290799660606`*^9, 3.6157291086907034`*^9}, {
  3.6162603720804358`*^9, 3.6162603901804714`*^9}, {3.6162606348114634`*^9, 
  3.6162606666952868`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", "LevelSetThreshold", "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "LevelSetThreshold", ",", "BottomThreshold", ",", "TopThreshold", ",", 
     "StepSize"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616162358042825*^9, 3.6161623594429054`*^9}, 
   3.6161623907446957`*^9}],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`LevelSetThreshold$$ = 0., Typeset`show$$ = 
    True, Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`LevelSetThreshold$$], -0.4, 0.19999999999999996`, 
      0.01}}, Typeset`size$$ = {504., {245., 255.}}, Typeset`update$$ = 0, 
    Typeset`initDone$$, Typeset`skipInitDone$$ = 
    True, $CellContext`LevelSetThreshold$1509$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`LevelSetThreshold$$ = -0.4}, 
      "ControllerVariables" :> {
        Hold[$CellContext`LevelSetThreshold$$, \
$CellContext`LevelSetThreshold$1509$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> ContourPlot[
        $CellContext`f[$CellContext`x, $CellContext`y], {$CellContext`x, 0, 
         1}, {$CellContext`y, 0, 1}, 
        Contours -> {$CellContext`LevelSetThreshold$$}], 
      "Specifications" :> {{$CellContext`LevelSetThreshold$$, -0.4, 
         0.19999999999999996`, 0.01}}, "Options" :> {}, 
      "DefaultOptions" :> {}],
     ImageSizeCache->{569., {313., 322.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellChangeTimes->{3.6157287875493355`*^9, 3.6161623607409797`*^9, 
  3.616162393918877*^9, 3.616259622055537*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tutorial 3: Making Persistence Diagrams", "Section",
 CellChangeTimes->{{3.6156494152402377`*^9, 3.615649417132346*^9}, {
  3.6157283873524456`*^9, 3.6157283925277414`*^9}}],

Cell[CellGroupData[{

Cell["A Filtration using Sub-level Sets", "Subsection",
 CellChangeTimes->{{3.6157285665596952`*^9, 3.6157285686138134`*^9}, {
  3.6157285987155347`*^9, 3.615728603644817*^9}}],

Cell[TextData[{
 "We set the directory ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " looks for files to be where ever the current Notebook is located."
}], "Text",
 CellChangeTimes->{{3.615644099682205*^9, 3.6156441479429655`*^9}, {
   3.6156490038487077`*^9, 3.615649004625752*^9}, 3.61755869424778*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}]], "Input",
 CellChangeTimes->{3.615644308256135*^9}],

Cell[TextData[{
 "Since the object we are studying is a subset of the plane, it will have \
trivial two-dimensional homology.\nHence, we are only interested in the zero \
and one dimensional homology.\n\nPlace the files \
\[OpenCurlyDoubleQuote]real_output_0.txt\[CloseCurlyDoubleQuote] and \
\[OpenCurlyDoubleQuote]real_output_1.txt\[CloseCurlyDoubleQuote] (produced by \
your modified Tutorial.cpp program) in the same folder as this notebook. \nWe \
import the data into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the following command."
}], "Text",
 CellChangeTimes->{{3.600541231105734*^9, 3.600541240483325*^9}, {
   3.600541272217286*^9, 3.600541332001134*^9}, {3.6161745398651934`*^9, 
   3.6161745562651315`*^9}, {3.61626371867385*^9, 3.6162637205879593`*^9}, {
   3.616264053404996*^9, 3.616264066310734*^9}, {3.6175586832811527`*^9, 
   3.6175587733003016`*^9}, {3.617558821850078*^9, 3.6175588712289023`*^9}, {
   3.617558914427373*^9, 3.6175589382277346`*^9}, {3.617559011154906*^9, 
   3.617559011521927*^9}, {3.617625357154318*^9, 3.617625375170349*^9}, {
   3.6176386591411486`*^9, 3.6176387220397463`*^9}, 3.6176388166921597`*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"SublevelBetti0", "=", 
  RowBox[{"Import", "[", 
   RowBox[{"\"\<real_output_0.txt\>\"", ",", "\"\<Table\>\""}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SublevelBetti1", "=", 
  RowBox[{"Import", "[", 
   RowBox[{"\"\<real_output_1.txt\>\"", ",", "\"\<Table\>\""}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6175584247653666`*^9, 3.617558428726593*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.16`"}], ",", 
     RowBox[{"-", "0.12`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.23`"}], ",", 
     RowBox[{"-", "0.1`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.3`"}], ",", 
     RowBox[{"-", "0.09`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.21`"}], ",", 
     RowBox[{"-", "0.08`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.08`"}], ",", 
     RowBox[{"-", "0.03`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.32`"}], ",", 
     RowBox[{"-", "0.42`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.617558627032935*^9, 3.6175588837356176`*^9, 
  3.617558941480921*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.05`", ",", "0.1`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.01`"}], ",", "0.12`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.617558627032935*^9, 3.6175588837356176`*^9, 
  3.6175589415089226`*^9}]
}, Open  ]],

Cell["\<\
We can now graph our two persistence diagrams. 
For reading sub-level set persistence diagrams, you should read the graph \
from left-to-right and from bottom-to-top.\
\>", "Text",
 CellChangeTimes->{{3.61626411851972*^9, 3.6162641781671314`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SubB0", "=", 
   RowBox[{"ListPlot", "[", "SublevelBetti0", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{"x", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "BottomThreshold", ",", "TopThreshold"}], "}"}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", ".4"}], ",", 
       RowBox[{"-", ".4"}]}], "}"}]}]}], "]"}], ";"}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"%", ",", "SubB0"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SubB1", "=", 
   RowBox[{"ListPlot", "[", "SublevelBetti1", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{"x", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "BottomThreshold", ",", "TopThreshold"}], "}"}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", ".4"}], ",", 
       RowBox[{"-", ".4"}]}], "}"}]}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"%", ",", "SubB1"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6175588923151083`*^9, 3.617558894650242*^9}, {
  3.6175591995846834`*^9, 3.6175592178377275`*^9}}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
     1.], LineBox[CompressedData["
1:eJxFxXswE3AcAPBV4oSuHIeuzMwfaeRxqtW6m85ynUxuR4tG5HUj14muoilR
J40yjBmZxXReyauxmDXNkmQq2zxOcfO40448Vumo/vD9/fG5D+7yVVrMdgwG
Q/3n//ekyWweTyvd2lNkIy7ioqfZkun8AjQldAcpJxdtvMqZT81A5xxq9mUw
0ZyiRQz2OFqYkHi9SqOB0xupoSkmaLr7uAWOooZtOwgVwqwRuFAT6TLf+gW2
OOabbb78GfZx2ft1xQEdcbDcupn2Cb4Zpbbz5wzD1CM/7epkKhjr2zc3szEE
a8LoRQYyutS2LK302yDMWtYxN8M/wJr1P+f0kvfwNkvu0gOHfhgveuSuFSjh
7riAuDZMH0yrSSSdvf0WDrcKNUlQy2HiY2We14k3sFl2QcsrcQ/cwKALHrp2
w0erva1scBKYYXp404AXw9eUbdoFXSucmrmmD5p8Cc94PputdGuA93AjJ72l
Ihi7lO+v6BLA9kZ6j9mUQjiXQEw4UHIX3ggkBE+uXyFvTUlnDbTnPYErhEly
J0EZXO66waocrYJ3clROfmu1cOmlQX31rSb4FJ83ZtnRAq+N1uWay9rhTik+
8IdpJzyVXczP8umCzQnOIyrHHtgtuiOW3iuDQ7DfQ/B+crh14Gn+aWUvXOyf
qZqIVcAZAWYpr+f74PELxLm0RTQxKoZBMqAXb0gpnUZKOLwy2VqMRZNWxtpb
gtGrvPpftTJ0YJU2MV6Jrm80nnL+iI6WR/Q/H0cPL1jxRQb0CzLrpND1HbzL
r7Yp0gsdG6R2wpHQ9kwPC8EZNJujmyiPQs+VWdLC4tE+NWTF/iT0bwmvkZ+O
Pq9QOF68j24eWubuY6N3jzmYjXLQTB31Do+H/gsNcZir
      "]]}}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.019444444444444445`],
      AbsoluteThickness[1.6], 
     PointBox[{{-0.16, -0.12}, {-0.23, -0.1}, {-0.3, -0.09}, {-0.21, -0.08}, \
{-0.08, -0.03}, {-0.32, -0.42}}]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-0.4, -0.4},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{-0.4, 0.19999999999999996`}, {-0.4, 0.199999987755102}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.615571719960334*^9, 3.615571762343758*^9}, {
   3.615571894488316*^9, 3.615571940270935*^9}, 3.6155720625729303`*^9, 
   3.6155725334488626`*^9, 3.615572609682223*^9, 3.6155732719851046`*^9, 
   3.6156443387878814`*^9, 3.615650498835216*^9, {3.6162642046536465`*^9, 
   3.616264225240824*^9}, 3.6175588971893873`*^9, 3.6175593307301846`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
     1.], LineBox[CompressedData["
1:eJxFxXswE3AcAPBV4oSuHIeuzMwfaeRxqtW6m85ynUxuR4tG5HUj14muoilR
J40yjBmZxXReyauxmDXNkmQq2zxOcfO40448Vumo/vD9/fG5D+7yVVrMdgwG
Q/3n//ekyWweTyvd2lNkIy7ioqfZkun8AjQldAcpJxdtvMqZT81A5xxq9mUw
0ZyiRQz2OFqYkHi9SqOB0xupoSkmaLr7uAWOooZtOwgVwqwRuFAT6TLf+gW2
OOabbb78GfZx2ft1xQEdcbDcupn2Cb4Zpbbz5wzD1CM/7epkKhjr2zc3szEE
a8LoRQYyutS2LK302yDMWtYxN8M/wJr1P+f0kvfwNkvu0gOHfhgveuSuFSjh
7riAuDZMH0yrSSSdvf0WDrcKNUlQy2HiY2We14k3sFl2QcsrcQ/cwKALHrp2
w0erva1scBKYYXp404AXw9eUbdoFXSucmrmmD5p8Cc94PputdGuA93AjJ72l
Ihi7lO+v6BLA9kZ6j9mUQjiXQEw4UHIX3ggkBE+uXyFvTUlnDbTnPYErhEly
J0EZXO66waocrYJ3clROfmu1cOmlQX31rSb4FJ83ZtnRAq+N1uWay9rhTik+
8IdpJzyVXczP8umCzQnOIyrHHtgtuiOW3iuDQ7DfQ/B+crh14Gn+aWUvXOyf
qZqIVcAZAWYpr+f74PELxLm0RTQxKoZBMqAXb0gpnUZKOLwy2VqMRZNWxtpb
gtGrvPpftTJ0YJU2MV6Jrm80nnL+iI6WR/Q/H0cPL1jxRQb0CzLrpND1HbzL
r7Yp0gsdG6R2wpHQ9kwPC8EZNJujmyiPQs+VWdLC4tE+NWTF/iT0bwmvkZ+O
Pq9QOF68j24eWubuY6N3jzmYjXLQTB31Do+H/gsNcZir
      "]]}}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.019444444444444445`],
      AbsoluteThickness[1.6], PointBox[{{0.05, 0.1}, {-0.01, 0.12}}]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-0.4, -0.4},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{-0.4, 0.19999999999999996`}, {-0.4, 0.199999987755102}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.615571719960334*^9, 3.615571762343758*^9}, {
   3.615571894488316*^9, 3.615571940270935*^9}, 3.6155720625729303`*^9, 
   3.6155725334488626`*^9, 3.615572609682223*^9, 3.6155732719851046`*^9, 
   3.6156443387878814`*^9, 3.615650498835216*^9, {3.6162642046536465`*^9, 
   3.616264225240824*^9}, 3.6175588971893873`*^9, 3.617559330788188*^9}]
}, Open  ]],

Cell["\<\
You can now copy the persistence diagrams and put them side-by-side with our \
graph of the sub-level set (the darker region).\
\>", "Text",
 CellChangeTimes->{{3.6157293863535852`*^9, 3.6157294339393067`*^9}, {
  3.615729511878765*^9, 3.6157295168200474`*^9}, {3.617558955398717*^9, 
  3.617558970699592*^9}, {3.6176388313309975`*^9, 3.6176388363552847`*^9}}],

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", "LevelSetThreshold", "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "LevelSetThreshold", ",", "BottomThreshold", ",", "TopThreshold", ",", 
     "StepSize"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.615650523193609*^9}],

Cell[BoxData[
 RowBox[{
  TagBox[
   StyleBox[
    DynamicModuleBox[{$CellContext`LevelSetThreshold$$ = -0.4, 
     Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
     Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
     1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{
       Hold[$CellContext`LevelSetThreshold$$], -0.4, 0.2, 0.01}}, 
     Typeset`size$$ = {405., {194., 205.}}, Typeset`update$$ = 0, 
     Typeset`initDone$$, Typeset`skipInitDone$$ = 
     True, $CellContext`LevelSetThreshold$14942$$ = 0}, 
     DynamicBox[Manipulate`ManipulateBoxes[
      1, StandardForm, 
       "Variables" :> {$CellContext`LevelSetThreshold$$ = -0.4}, 
       "ControllerVariables" :> {
         Hold[$CellContext`LevelSetThreshold$$, \
$CellContext`LevelSetThreshold$14942$$, 0]}, 
       "OtherVariables" :> {
        Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
         Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
         Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
         Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> ContourPlot[
         $CellContext`f[$CellContext`x, $CellContext`y], {$CellContext`x, 0, 
          1}, {$CellContext`y, 0, 1}, 
         Contours -> {$CellContext`LevelSetThreshold$$}], 
       "Specifications" :> {{$CellContext`LevelSetThreshold$$, -0.4, 0.2, 
          0.01}}, "Options" :> {}, "DefaultOptions" :> {}],
      ImageSizeCache->{509., {262., 271.}},
      SingleEvaluation->True],
     Deinitialization:>None,
     DynamicModuleValues:>{},
     SynchronousInitialization->True,
     UnsavedVariables:>{Typeset`initDone$$},
     UntrackedVariables:>{Typeset`size$$}], "Manipulate",
    Deployed->True,
    StripOnInput->False],
   Manipulate`InterpretManipulate[1]], 
  GraphicsBox[{{{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJxFxW0w03EcAPCVjnPmDsctPRwbL/IYTpesu+ksuYU6cUMoeYrlOofrSvRA
nTRimvo3NIvpKCcbjcWsaf5JspRtxq1yy9xppzxMcVQvfH8vPvchn7sYnb6d
QCBE/vP/m7I8NoZpZVsHCEkSbi16hi2dqa5B0xMsqOUVaMtlztyVG+hyz86w
xCw0h7tAcDmEFrByCpo0Gri4PTIh3wrN9JuyJdPVMKnH67GgdAKu0aR4z4k/
w8SDYWXExU/wEW/7L0uu6KR99U6d0eNwQaraOYLzEWYcWHVuk6vg3WFDxu8b
Y/B4EpNrpqGrdtYVPvo6CuctGrI2k9/Do2vrJ0zSd7DZvvbnbddh2Fl410/L
x2FxZlRmF2EIDm/JoR6/+gaOcUywYqkVsO89vDIw+DW8raxG9FIyAGOJTP4d
n36Y0hziSCJL4XBr302zmwROxru08wYxnFGyYorRv4D7A57MNu5/Dhu5KfoQ
mRA2L1RHKPv48IiFyX82/z486xnE2vvwOtxwyitWv3aBtjVWXDTSXVkFLwty
Fe78Ohj32ShqnGyC/Tgqd8ZKK9xxZtTUfLkDZvIwnUOPCCbq2iqI8m54TOZ2
8pd1L7xa9oBXGtoHk7w8JlSUATg4rSeDOSiH01x+xLsxFLBspKH6KD4ICyJK
VNMZSpgdZZP/am4INsYFGQsX0KGp6YlUM/rPJRm9dwcOn2/Mc5K4oOlLum5R
LHode/a7VY6Oa9LmZONoUbvlN48P6GzF2eGnU2jNvCNPaEaLaUWHBT5vYTtG
a0dKIJoVo3YnU9GULH9bfji6imOYrk9Fz9c5RCdlo4+10JR7ctEbUqydV4yO
Vyopp2+hxWOLtbvYaDudq80kB80yRF7DMPRfhCOfsw==
       "]]}}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.019444444444444445`], AbsoluteThickness[1.6], 
      PointBox[{{-0.16000000000000003`, -0.12}, {-0.23, \
-0.10000000000000003`}, {-0.30000000000000004`, -0.09000000000000002}, \
{-0.21000000000000002`, -0.08000000000000002}, {-0.08000000000000002, \
-0.030000000000000027`}, {-0.32, -0.42000000000000004`}}]}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{-0.4, -0.4},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->{343.0952380952358, Automatic},
   Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
   PlotRange->{{-0.4, 0.2}, {-0.4, 0.19999998775510205`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}], 
  GraphicsBox[{{{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJxFxW0w03EcAPCVjnPmDsctPRwbL/IYTpesu+ksuYU6cUMoeYrlOofrSvRA
nTRimvo3NIvpKCcbjcWsaf5JspRtxq1yy9xppzxMcVQvfH8vPvchn7sYnb6d
QCBE/vP/m7I8NoZpZVsHCEkSbi16hi2dqa5B0xMsqOUVaMtlztyVG+hyz86w
xCw0h7tAcDmEFrByCpo0Gri4PTIh3wrN9JuyJdPVMKnH67GgdAKu0aR4z4k/
w8SDYWXExU/wEW/7L0uu6KR99U6d0eNwQaraOYLzEWYcWHVuk6vg3WFDxu8b
Y/B4EpNrpqGrdtYVPvo6CuctGrI2k9/Do2vrJ0zSd7DZvvbnbddh2Fl410/L
x2FxZlRmF2EIDm/JoR6/+gaOcUywYqkVsO89vDIw+DW8raxG9FIyAGOJTP4d
n36Y0hziSCJL4XBr302zmwROxru08wYxnFGyYorRv4D7A57MNu5/Dhu5KfoQ
mRA2L1RHKPv48IiFyX82/z486xnE2vvwOtxwyitWv3aBtjVWXDTSXVkFLwty
Fe78Ohj32ShqnGyC/Tgqd8ZKK9xxZtTUfLkDZvIwnUOPCCbq2iqI8m54TOZ2
8pd1L7xa9oBXGtoHk7w8JlSUATg4rSeDOSiH01x+xLsxFLBspKH6KD4ICyJK
VNMZSpgdZZP/am4INsYFGQsX0KGp6YlUM/rPJRm9dwcOn2/Mc5K4oOlLum5R
LHode/a7VY6Oa9LmZONoUbvlN48P6GzF2eGnU2jNvCNPaEaLaUWHBT5vYTtG
a0dKIJoVo3YnU9GULH9bfji6imOYrk9Fz9c5RCdlo4+10JR7ctEbUqydV4yO
Vyopp2+hxWOLtbvYaDudq80kB80yRF7DMPRfhCOfsw==
       "]]}}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.019444444444444445`], AbsoluteThickness[1.6], 
      PointBox[{{0.04999999999999999, 
       0.09999999999999998}, {-0.010000000000000009`, 0.12}}]}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{-0.4, -0.4},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->{353.99999999999477`, Automatic},
   Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
   PlotRange->{{-0.4, 0.2}, {-0.4, 0.19999998775510205`}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}]}]], "Input",
 CellChangeTimes->{{3.6157294379405355`*^9, 3.6157294518513317`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Filtration using Super-level Sets", "Subsection",
 CellChangeTimes->{{3.6157285145517206`*^9, 3.615728552684902*^9}, {
  3.6157286099441767`*^9, 3.615728619869745*^9}}],

Cell[TextData[{
 "Place the files \[OpenCurlyDoubleQuote]real_output_0.txt\
\[CloseCurlyDoubleQuote] and \[OpenCurlyDoubleQuote]real_output_1.txt\
\[CloseCurlyDoubleQuote] (produced by your modified Tutorial.cpp program) in \
the same folder as this Notebook. \nThen we import their data into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " with the following command."
}], "Text",
 CellChangeTimes->{{3.6156441794287663`*^9, 3.61564429418433*^9}, {
   3.6162633137566905`*^9, 3.6162633451764874`*^9}, 3.617559006032613*^9, {
   3.6175591395792513`*^9, 3.6175591425234194`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"SuperlevelBetti0", "=", 
  RowBox[{"Import", "[", 
   RowBox[{"\"\<real_output_0.txt\>\"", ",", "\"\<Table\>\""}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"SuperlevelBetti1", "=", 
  RowBox[{"Import", "[", 
   RowBox[{"\"\<real_output_1.txt\>\"", ",", "\"\<Table\>\""}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6175584247653666`*^9, 3.617558428726593*^9}, {
  3.61755909970197*^9, 3.617559103311177*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "5.55112`*^-17"}], ",", 
     RowBox[{"-", "0.02`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.03`"}], ",", 
     RowBox[{"-", "0.04`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.15`", ",", 
     RowBox[{"-", "0.05`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.11`", ",", 
     RowBox[{"-", "0.1`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.09`", ",", 
     RowBox[{"-", "0.12`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.18`", ",", "0.22`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.61755915197296*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.23`"}], ",", 
     RowBox[{"-", "0.24`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.13`"}], ",", 
     RowBox[{"-", "0.33`"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.617559152003962*^9}]
}, Open  ]],

Cell["\<\
We can now graph our two persistence diagrams. 
For reading super-level set persistence diagrams, you should read \
right-to-left and top-to-bottom.\
\>", "Text",
 CellChangeTimes->{
  3.6162630518087077`*^9, {3.6162632955906515`*^9, 3.616263308648398*^9}, {
   3.6162633564371314`*^9, 3.6162633788084106`*^9}, {3.616264157379943*^9, 
   3.616264158122985*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SupB0", "=", 
   RowBox[{"ListPlot", "[", "SuperlevelBetti0", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{"x", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "BottomThreshold", ",", "TopThreshold"}], "}"}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"TopThreshold", ",", "TopThreshold"}], "}"}]}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"%", ",", "SupB0"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SupB1", "=", 
   RowBox[{"ListPlot", "[", "SuperlevelBetti1", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{"x", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "BottomThreshold", ",", "TopThreshold"}], "}"}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"TopThreshold", ",", "TopThreshold"}], "}"}]}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"%", ",", "SupB1"}], "]"}]}], "Input"],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
     1.], LineBox[CompressedData["
1:eJxFxXswE3AcAPBV4oSuHIeuzMwfaeRxqtW6m85ynUxuR4tG5HUj14muoilR
J40yjBmZxXReyauxmDXNkmQq2zxOcfO40448Vumo/vD9/fG5D+7yVVrMdgwG
Q/3n//ekyWweTyvd2lNkIy7ioqfZkun8AjQldAcpJxdtvMqZT81A5xxq9mUw
0ZyiRQz2OFqYkHi9SqOB0xupoSkmaLr7uAWOooZtOwgVwqwRuFAT6TLf+gW2
OOabbb78GfZx2ft1xQEdcbDcupn2Cb4Zpbbz5wzD1CM/7epkKhjr2zc3szEE
a8LoRQYyutS2LK302yDMWtYxN8M/wJr1P+f0kvfwNkvu0gOHfhgveuSuFSjh
7riAuDZMH0yrSSSdvf0WDrcKNUlQy2HiY2We14k3sFl2QcsrcQ/cwKALHrp2
w0erva1scBKYYXp404AXw9eUbdoFXSucmrmmD5p8Cc94PputdGuA93AjJ72l
Ihi7lO+v6BLA9kZ6j9mUQjiXQEw4UHIX3ggkBE+uXyFvTUlnDbTnPYErhEly
J0EZXO66waocrYJ3clROfmu1cOmlQX31rSb4FJ83ZtnRAq+N1uWay9rhTik+
8IdpJzyVXczP8umCzQnOIyrHHtgtuiOW3iuDQ7DfQ/B+crh14Gn+aWUvXOyf
qZqIVcAZAWYpr+f74PELxLm0RTQxKoZBMqAXb0gpnUZKOLwy2VqMRZNWxtpb
gtGrvPpftTJ0YJU2MV6Jrm80nnL+iI6WR/Q/H0cPL1jxRQb0CzLrpND1HbzL
r7Yp0gsdG6R2wpHQ9kwPC8EZNJujmyiPQs+VWdLC4tE+NWTF/iT0bwmvkZ+O
Pq9QOF68j24eWubuY6N3jzmYjXLQTB31Do+H/gsNcZir
      "]]}}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.019444444444444445`],
      AbsoluteThickness[1.6], 
     PointBox[{{-5.55112*^-17, -0.02}, {-0.03, -0.04}, {0.15, -0.05}, {
      0.11, -0.1}, {0.09, -0.12}, {0.18, 0.22}}]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0.19999999999999996`, 0.19999999999999996`},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{-0.4, 0.19999999999999996`}, {-0.39999998775510204`, 
   0.19999999999999996`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.617559243074171*^9, 3.6175592536257744`*^9}}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
     1.], LineBox[CompressedData["
1:eJxFxXswE3AcAPBV4oSuHIeuzMwfaeRxqtW6m85ynUxuR4tG5HUj14muoilR
J40yjBmZxXReyauxmDXNkmQq2zxOcfO40448Vumo/vD9/fG5D+7yVVrMdgwG
Q/3n//ekyWweTyvd2lNkIy7ioqfZkun8AjQldAcpJxdtvMqZT81A5xxq9mUw
0ZyiRQz2OFqYkHi9SqOB0xupoSkmaLr7uAWOooZtOwgVwqwRuFAT6TLf+gW2
OOabbb78GfZx2ft1xQEdcbDcupn2Cb4Zpbbz5wzD1CM/7epkKhjr2zc3szEE
a8LoRQYyutS2LK302yDMWtYxN8M/wJr1P+f0kvfwNkvu0gOHfhgveuSuFSjh
7riAuDZMH0yrSSSdvf0WDrcKNUlQy2HiY2We14k3sFl2QcsrcQ/cwKALHrp2
w0erva1scBKYYXp404AXw9eUbdoFXSucmrmmD5p8Cc94PputdGuA93AjJ72l
Ihi7lO+v6BLA9kZ6j9mUQjiXQEw4UHIX3ggkBE+uXyFvTUlnDbTnPYErhEly
J0EZXO66waocrYJ3clROfmu1cOmlQX31rSb4FJ83ZtnRAq+N1uWay9rhTik+
8IdpJzyVXczP8umCzQnOIyrHHtgtuiOW3iuDQ7DfQ/B+crh14Gn+aWUvXOyf
qZqIVcAZAWYpr+f74PELxLm0RTQxKoZBMqAXb0gpnUZKOLwy2VqMRZNWxtpb
gtGrvPpftTJ0YJU2MV6Jrm80nnL+iI6WR/Q/H0cPL1jxRQb0CzLrpND1HbzL
r7Yp0gsdG6R2wpHQ9kwPC8EZNJujmyiPQs+VWdLC4tE+NWTF/iT0bwmvkZ+O
Pq9QOF68j24eWubuY6N3jzmYjXLQTB31Do+H/gsNcZir
      "]]}}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.019444444444444445`],
      AbsoluteThickness[1.6], 
     PointBox[{{-0.23, -0.24}, {-0.13, -0.33}}]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0.19999999999999996`, 0.19999999999999996`},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
  PlotRange->{{-0.4, 0.19999999999999996`}, {-0.39999998775510204`, 
   0.19999999999999996`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.617559243074171*^9, 3.6175592536847773`*^9}}]
}, Open  ]],

Cell["\<\
Again, it is nice to compare the persistence diagrams side-by-side with the \
graphs of the super-level sets (the lighter regions).\
\>", "Text",
 CellChangeTimes->{{3.615728817733062*^9, 3.6157289162066946`*^9}, {
   3.6157292286415644`*^9, 3.6157292888870106`*^9}, {3.615729483806159*^9, 
   3.615729493661723*^9}, 3.616263044436286*^9, {3.6162633956363735`*^9, 
   3.6162634342315807`*^9}, {3.6175589772279654`*^9, 
   3.6175589802791395`*^9}, {3.617638841942604*^9, 3.61763884571682*^9}}],

Cell[BoxData[
 RowBox[{"Manipulate", "[", 
  RowBox[{
   RowBox[{"ContourPlot", "[", 
    RowBox[{
     RowBox[{"f", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", "LevelSetThreshold", "}"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"LevelSetThreshold", ",", "TopThreshold"}], "}"}], ",", 
     "BottomThreshold", ",", "TopThreshold", ",", "StepSize"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.6155696349820795`*^9, 3.6155696638897333`*^9}, {
   3.6155698799270897`*^9, 3.6155699587525983`*^9}, {3.615571467750908*^9, 
   3.615571476630416*^9}, 3.6155717019503036`*^9, {3.6155717411705465`*^9, 
   3.615571744637745*^9}, {3.615644710496142*^9, 3.6156447507294436`*^9}, 
   3.615645835563492*^9}],

Cell[BoxData[
 RowBox[{
  TagBox[
   StyleBox[
    DynamicModuleBox[{$CellContext`LevelSetThreshold$$ = 0.19999999999999996`,
      Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
     Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
     1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{{
        Hold[$CellContext`LevelSetThreshold$$], 0.19999999999999996`}, -0.4, 
       0.19999999999999996`, 0.01}}, Typeset`size$$ = {470., {222., 232.}}, 
     Typeset`update$$ = 0, Typeset`initDone$$, Typeset`skipInitDone$$ = 
     True, $CellContext`LevelSetThreshold$7795$$ = 0}, 
     DynamicBox[Manipulate`ManipulateBoxes[
      1, StandardForm, 
       "Variables" :> {$CellContext`LevelSetThreshold$$ = 
         0.19999999999999996`}, "ControllerVariables" :> {
         Hold[$CellContext`LevelSetThreshold$$, \
$CellContext`LevelSetThreshold$7795$$, 0]}, 
       "OtherVariables" :> {
        Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
         Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
         Typeset`specs$$, Typeset`size$$, Typeset`update$$, 
         Typeset`initDone$$, Typeset`skipInitDone$$}, "Body" :> ContourPlot[
         $CellContext`f[$CellContext`x, $CellContext`y], {$CellContext`x, 0, 
          1}, {$CellContext`y, 0, 1}, 
         Contours -> {$CellContext`LevelSetThreshold$$}], 
       "Specifications" :> {{{$CellContext`LevelSetThreshold$$, 
           0.19999999999999996`}, -0.4, 0.19999999999999996`, 0.01}}, 
       "Options" :> {}, "DefaultOptions" :> {}],
      ImageSizeCache->{535., {290., 299.}},
      SingleEvaluation->True],
     Deinitialization:>None,
     DynamicModuleValues:>{},
     SynchronousInitialization->True,
     UnsavedVariables:>{Typeset`initDone$$},
     UntrackedVariables:>{Typeset`size$$}], "Manipulate",
    Deployed->True,
    StripOnInput->False],
   Manipulate`InterpretManipulate[1]], 
  GraphicsBox[{{{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJxFxW0w03EcAPCVjnPmDsctPRwbL/IYTpesu+ksuYU6cUMoeYrlOofrSvRA
nTRimvo3NIvpKCcbjcWsaf5JspRtxq1yy9xppzxMcVQvfH8vPvchn7sYnb6d
QCBE/vP/m7I8NoZpZVsHCEkSbi16hi2dqa5B0xMsqOUVaMtlztyVG+hyz86w
xCw0h7tAcDmEFrByCpo0Gri4PTIh3wrN9JuyJdPVMKnH67GgdAKu0aR4z4k/
w8SDYWXExU/wEW/7L0uu6KR99U6d0eNwQaraOYLzEWYcWHVuk6vg3WFDxu8b
Y/B4EpNrpqGrdtYVPvo6CuctGrI2k9/Do2vrJ0zSd7DZvvbnbddh2Fl410/L
x2FxZlRmF2EIDm/JoR6/+gaOcUywYqkVsO89vDIw+DW8raxG9FIyAGOJTP4d
n36Y0hziSCJL4XBr302zmwROxru08wYxnFGyYorRv4D7A57MNu5/Dhu5KfoQ
mRA2L1RHKPv48IiFyX82/z486xnE2vvwOtxwyitWv3aBtjVWXDTSXVkFLwty
Fe78Ohj32ShqnGyC/Tgqd8ZKK9xxZtTUfLkDZvIwnUOPCCbq2iqI8m54TOZ2
8pd1L7xa9oBXGtoHk7w8JlSUATg4rSeDOSiH01x+xLsxFLBspKH6KD4ICyJK
VNMZSpgdZZP/am4INsYFGQsX0KGp6YlUM/rPJRm9dwcOn2/Mc5K4oOlLum5R
LHode/a7VY6Oa9LmZONoUbvlN48P6GzF2eGnU2jNvCNPaEaLaUWHBT5vYTtG
a0dKIJoVo3YnU9GULH9bfji6imOYrk9Fz9c5RCdlo4+10JR7ctEbUqydV4yO
Vyopp2+hxWOLtbvYaDudq80kB80yRF7DMPRfhCOfsw==
       "]]}}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.019444444444444445`], AbsoluteThickness[1.6], 
      PointBox[{{0., -0.01999999999999999}, {-0.03, -0.03999999999999998}, {
       0.15000000000000002`, -0.04999999999999999}, {
       0.11000000000000001`, -0.09999999999999998}, {
       0.09000000000000001, -0.12}, {0.18000000000000002`, 0.22}}]}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0.2, 0.2},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->{359.999999999999, Automatic},
   Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
   PlotRange->{{-0.4, 0.2}, {-0.39999998775510204`, 0.2}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}], 
  GraphicsBox[{{{}, {}, 
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJxFxW0w03EcAPCVjnPmDsctPRwbL/IYTpesu+ksuYU6cUMoeYrlOofrSvRA
nTRimvo3NIvpKCcbjcWsaf5JspRtxq1yy9xppzxMcVQvfH8vPvchn7sYnb6d
QCBE/vP/m7I8NoZpZVsHCEkSbi16hi2dqa5B0xMsqOUVaMtlztyVG+hyz86w
xCw0h7tAcDmEFrByCpo0Gri4PTIh3wrN9JuyJdPVMKnH67GgdAKu0aR4z4k/
w8SDYWXExU/wEW/7L0uu6KR99U6d0eNwQaraOYLzEWYcWHVuk6vg3WFDxu8b
Y/B4EpNrpqGrdtYVPvo6CuctGrI2k9/Do2vrJ0zSd7DZvvbnbddh2Fl410/L
x2FxZlRmF2EIDm/JoR6/+gaOcUywYqkVsO89vDIw+DW8raxG9FIyAGOJTP4d
n36Y0hziSCJL4XBr302zmwROxru08wYxnFGyYorRv4D7A57MNu5/Dhu5KfoQ
mRA2L1RHKPv48IiFyX82/z486xnE2vvwOtxwyitWv3aBtjVWXDTSXVkFLwty
Fe78Ohj32ShqnGyC/Tgqd8ZKK9xxZtTUfLkDZvIwnUOPCCbq2iqI8m54TOZ2
8pd1L7xa9oBXGtoHk7w8JlSUATg4rSeDOSiH01x+xLsxFLBspKH6KD4ICyJK
VNMZSpgdZZP/am4INsYFGQsX0KGp6YlUM/rPJRm9dwcOn2/Mc5K4oOlLum5R
LHode/a7VY6Oa9LmZONoUbvlN48P6GzF2eGnU2jNvCNPaEaLaUWHBT5vYTtG
a0dKIJoVo3YnU9GULH9bfji6imOYrk9Fz9c5RCdlo4+10JR7ctEbUqydV4yO
Vyopp2+hxWOLtbvYaDudq80kB80yRF7DMPRfhCOfsw==
       "]]}}, {{}, 
     {RGBColor[0.368417, 0.506779, 0.709798], PointSize[
      0.019444444444444445`], AbsoluteThickness[1.6], 
      PointBox[{{-0.22999999999999998`, -0.24}, {-0.13, -0.33}}]}, {}}},
   AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
   Axes->{True, True},
   AxesLabel->{None, None},
   AxesOrigin->{0.2, 0.2},
   DisplayFunction->Identity,
   Frame->{{False, False}, {False, False}},
   FrameLabel->{{None, None}, {None, None}},
   FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
   GridLines->{None, None},
   GridLinesStyle->Directive[
     GrayLevel[0.5, 0.4]],
   ImageSize->{372.6666666666681, Automatic},
   Method->{"DefaultBoundaryStyle" -> Automatic, "ScalingFunctions" -> None},
   PlotRange->{{-0.4, 0.2}, {-0.39999998775510204`, 0.2}},
   PlotRangeClipping->True,
   PlotRangePadding->{{
      Scaled[0.02], 
      Scaled[0.02]}, {
      Scaled[0.05], 
      Scaled[0.05]}},
   Ticks->{Automatic, Automatic}]}]], "Input",
 CellChangeTimes->{{3.615729117683218*^9, 3.61572912068639*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.615731478984277*^9, 3.6157314886758313`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tutorial 4: Defining A New Function", "Section",
 CellChangeTimes->{{3.6157429152653947`*^9, 3.615742943215993*^9}, {
  3.615742983568301*^9, 3.6157429992361975`*^9}, {3.615743048743029*^9, 
  3.615743049793089*^9}, {3.6157442776703196`*^9, 3.615744281102516*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"20.0", 
    RowBox[{"(", 
     RowBox[{"y", "-", ".3"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"y", "-", ".7"}], ")"}]}], "+", 
   RowBox[{"20", 
    RowBox[{"(", 
     RowBox[{"x", "-", ".1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "-", ".4"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"x", "-", ".9"}], ")"}]}]}]}], "\[IndentingNewLine]", 
 RowBox[{"ContourPlot", "[", 
  RowBox[{
   RowBox[{"g", "[", 
    RowBox[{"x", ",", "y"}], "]"}], " ", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "Automatic"}]}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.615743052371237*^9, 3.6157431619275026`*^9}, {
   3.615743224660091*^9, 3.6157433242367864`*^9}, {3.6157433622399597`*^9, 
   3.6157433659241705`*^9}, 3.615743700438304*^9, 3.61574390632308*^9, {
   3.615744086554388*^9, 3.615744170778206*^9}, {3.615744288274926*^9, 
   3.6157444261388116`*^9}, {3.6157453872687855`*^9, 
   3.6157454013015876`*^9}, {3.615745574165475*^9, 3.615745602269082*^9}, {
   3.6161667599455137`*^9, 3.6161667602419147`*^9}, {3.616184087099264*^9, 
   3.616184281884405*^9}}],

Cell[BoxData[
 TemplateBox[{GraphicsBox[
    GraphicsComplexBox[CompressedData["
1:eJyEfQVYlM/zOCp+7O7uDmzFGrvF7g92Yyd2ix2IKGChiIEojYISB4ISSncd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     "], {{{
        EdgeForm[], 
        RGBColor[0.14800000000000002`, 0.33, 0.54], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxFmXmMltUVxt/3nWG275vve00LskoXNBgXEBQYlhmEsohLWlNJBNtYGWii
uGGpWmeAAdu0CERBFqtpxDapaVItArLooAiKgBap2ApKWVSgW6JWRItInx/n
mfjHnXvnvveee+5Zn3u+b950+7W3ZUmSvK4/FeqH1SbJULVX0iR5I0+SFzsl
yW+KSVJdE+MRWvdgZZI8qTX9taZSaxq08VRB+/X/ao2vE53Omr9We76m/oX6
JLlY3/ZqvLIuSR5XW8o5ojdOtIZozxV1QfMi/X88ibnJ6pfr7DGaS85Kkh/o
7KOic47a+2qH1Xqq3Ss6i7RmhPa06IwNOrdF+6bo/wdE8xNt36h1fdRKah9U
62x9G6w2Ut9/rLOb1Depb1T7lfbPSINeQf3D+pZVJcnCyhgvURtu2gvUENxj
dfE/43F5yGFkMfY97PXXiO5Vam9lQYN5aDJ3tdpibf+DZHWZzjyqvZO1br/G
3dWatX9hbfBPv0gt1dyQurgLMmusCZny//Nl6VJ9u/ova2IP64fXho75Vi36
izW+PQ2dMK5Ko1/i+R+Jn2+JZq7+bK1/VXOd1MZr/XrRzcX/NRpvrAiZ/db8
c49pdSHbvfr/SbVyGrq4BbvhThovkN0ME41FxVgzX+fOVL9btAerTdSaLlnQ
gg4ynm+dojPmu8GT1tyaBv+HNXdIrUXtSssc2a7SOXuysFvu9hT7k+j5vyYN
3bDnUo0b0xizF5kOsZyHlpJkgvQ6RP0wyWaC1ozTt3XS+2mNN+Qh46l14Uu0
Zo1vFr1E40e0Z57GbZLRCnxAd6/SvudFc5jO3yK6z2mcaH55HuvHF0OW02yr
n2r/cbV+ap2z8NcsDZ1ssF6W2Xee0Dkbq4LmT0X2f1r3mdolaehiXU3og72M
8V/4hf+bkrjbaLUGrZmehlzxdfjdaJ6HZzGG/iF9e0eth9o0zX+hfqBaq3Uy
JQm/WI6/qF9WG2N8BB1WZsHPeervU1uI/egOfXWXWbrTatE4lobu5qQRE/DT
27x3quYLxfC3s/KIK+gevSPLCZUh/7drrD/9P0n9BbVhH5x3Uu1+0anJwraw
qzXYVxbn9BTfKxVH2sVzb82d0txMrV+Uxn2/4buf8nig56F5h2nC5/Y0fGqG
xjs8Rj6X6fs2je/U+IYsYt1PkuDhPbVqtV6er3VMPGoeiBsHveZZrydOPpCG
btDLWsfQno6j75vOS2nIHPs/UBvxMdMdb9T/O9OIyfC43fY2Sd+urwhe/1gf
voxPV0j+2yWjXbLlQVmsn+G9OxxDpnvMfcumj1+vqAw6+B022Gw/OphHnB8o
+iPSyEPodJ3a58gjDT5esdyYO+F5fOVTj094Pfa/wP/P0/pzs/CL3mnE4xXm
odnzrMFGzredTFH7xPJEtoznJpHjltmeO+yb/Eq/1ON284P/XpfF3ZHB0Pqw
xe26a2VFxCN00SeLO/TynuM+a1BN0MQ++5o3ci257h9p5Is70oj1+At2h013
zNXYtqdmYfPc/aDtvNp2UpuFbZAXqr3+1uqQ/0Oi+YHXQPN88f6Evm0W75vS
iDXImdhJrseOyOVjHJ9mms+bNT3fd4D/z7wXfrJOsZ5csEJ7xnJuIebGev7X
xYjfcwqhG+i0JeEX8Mfdx4iHrbXho+R58AH29kgxeHsQrKJ8+S/x/6L61yTb
HWoN+vZn2fHL0uXr6j/U+hc0t0vr++m8b4u/s6S3X+n/nVq/ALxUjJi5pRy2
ur0mZAVOAi9dnQd2IJ/BB2ewZoDWPpxHTHpMNC5U/6rO3ZcGLztN50vTAX8t
VtvDfB75n/E409vh9W9UR44+nUUPZgLf4KMjwXZVgYvOYAjNXyr6c7Rvp+77
djF8YW8x8l6z8wL+Sx4gj6Nv5IOMa4rRv6a9czW+WHt/Vx+x+UyMVhuutduq
wl+JY+vVz1bbrT33YWvEkIqgi77eBEtJVhvz0PluY4PThcAy4B3sDh7Y07k+
MNjlxcBK4B8wFHGbmA1O+25N3J+cBZ4ZVxf5rykN38PvflYf+fD6Qtg2tOHr
GX1rdRxDRshqUCHiEzmdnLjDMY41lxrz4q9zdId29bPz8L9q+06V4zYy6p5F
7J6VBqYAw53Bb+J3Xm3IiJxL/iP3bRcPm/WtUvdtFc9LxcN3ipGjNvnOYE3O
JY+ib9aD5/9dHzHpuULI40XLhG/clfzVmMU8c3t01usav1kKXu43P4/WxTkz
fV/wG3Tx75mORWDFp2vDDjPx14TdlCKfz/BdoAHPfGvL403QVowYs9bxbJzu
1bsq9NRomZAzwQdncIL2fVEIjP913ih1gZW/J542lmOe7xX6NkRzJwvB7xDz
vK0UPLziu3JHzsGvkC1xDMzPncGjvBmI4cxvqY04Aw4+rzLsED+BBrSQ1Xt5
8D+4EHdgTKyfVxU2in2WSqHztlLsnWcewJH4YZpGPs2cQ8nV5GxwMjGj0XED
H1jtd8kB/f/3NN4UXfOIUcSq3cWwXXLs2WoTdZ8uxYinxNXfy0a+DzZU+2US
8uR9N9jYkG/gQ7AGGAVcccSxlxiMbJAR8XZyRfAJrphfE/PIDRyBL5O/+c46
7tJkmSLPv1of2CDxY5KxR5c8eOumfrh4nSwdNatvL4RcP64PnbxsOqyd6Lv0
L0UMXlmId89i527sabhjFG9DeCXm9i1FjvpFIfAzuiVGjKiK9ej3klLEmwPl
WMd63sH7Ko1x07Bv1uAXn+dxl+6S+aiKwNzIs802sLcQ/MFnJ63tVoz17DtW
/CrugpmRF7HrwlLw9Bf1s51LPtT6HsWQbS+feX1NyB4fx7/x90WWOXh1cxpv
IXITuay/sccwYxUwyypj6D1+PzWah8466yqtvbE+MNzNtldyAnyii39Wxzz2
DH7pa3xF/hrhc8GPLd47Wt9HqVVURQ4j1hI3sEf8m/Netg8iF95nH1eHTNE7
tjracarJ8udtRD4g7m2tD15mSd5rHK+ITcQsxvgDeGlDGrietwQ6uzeJ7095
DTLG/7DhbbZvbHu56wnYGO8P+GI/mBQsC156N403FG9bcObqNHBfzzz01UP9
vkL40H/LkT+P2NeokxxzLCVmnsFn5hGbgk8wPli/463DPdA1MttijNSaB7YB
4+SuA5TVd66N+V3ZVzaBPQywnYCZwYbMP2SZEPeR4QSfyzpwBngFbAjmmlUX
tgD2QS7wC1ZhL7KnDtBimbzju19RjPszZu6Hzjfg378VYt+jxXh7Qn+X32Tc
l7cS/okPviof6ZdFDQIsSq4kj1AXID/TT/P7i/zM222L9NCvMmyZ1s82faQY
/jBDtC+wD+KLxF5wA3gAvDfAsuJ9irzIc2OzwM0db5MBlu0Q1xyeLgf2BIuQ
m3mbnPD6j+rDJ84vxVukn/cyB7YmBj1g/20DD/sdwjpiEtgEbD/fMmYNOXGQ
sRY4nPhFbQEbXub3OzmQXEjO4m1b6VoM8m61TrEXciHyBx9R78Bvr8kjrl6V
hx2QByb7TXHEdsv50IMHcheYCsxJLCBuTHG9YoJz4/o8Yib5s9L6Yv3YYuSX
065V3uJ5MPhjzuPUJ6abJljwFsf2lkLg4+fK8e5AFshhn8cT/X48aZ/CxsBm
xNIKY1jG51i/YN2uWWBI3o/YPPZOLJmUhZ2Qow9bhsgNOba6fnLMvo5NrvcY
e+5je8B28OMm+zU2Ro0VuwT3Uc8l1mOz4A/WDTG+Inf3sG2APz+x3cDnQfPV
7LrEs16PnYD50BVyxI/I9TcUAs/Ul8LeiGN3JVFv5J1MDATvNTkes2+mfRAb
uMN3BDuz50x9shA59q1S5OrHTafFdT90S82aevWq+njrkb/J3djsSmNxYssI
1y17u7aMzpqyyIWMe7rGgr7+VI54QlwZ6vjPPO8G3g/YAzbD/9gduYNaFfmA
+vM8x945RWPaPPg9bP3eVwxb481xyDLHB8mH8z0mTvD2Zj/1pEXWb3+vx9/P
cw2L+eO2hbnO44ypY3xmO2XMuwqaxGryapPPazVvLa6JI0d0Cs/4MnIen8e4
1fX2Rsv5okLEhv/UR+wklpIT8fUm0+MtNca+AY1DlkNDFnkQ20NuvH2QXbcs
6uHMg23BZ+RX6gT4VG/rizG1snIpcDVYGz9bbRum7oJt9zLNdtskNoENg//J
yWAGcMLWcox/Xoj8tdlyG2w/ZS+4Cnw1r+BacV1g4315YJq388AmzB83BmY9
/N9WDvlPL4ftEFOwH9Y2mA75kby7y7F1n+se6BN+yBPYO7kBvya/kmfBYOCi
Br99hvqtTawgB27w+BzbMzLs5PcoOe6eLOr+yI86HrikxeePs83fnUUNusUx
G/q1jiHrLOexHqMX4swa00e36Lhke3zJsqUeudZ0eFMQZ6odD9Z6b4PjDzSr
TJM1o7Kgea91hN5bbUenbUtz88jZ1CeoDfANHqjpIMOPNP9UOeockwoRq++x
HMBw3PdO1zORD7VNavNglG6WEWsyywO5zzYfrO/k73ebzn7b4l2WeVfTPNvn
Mj7lOEVcotbO+58cSnwhzhAnu5gm5yNvZIR87jZ+4qyjpoPe02LU8beWQn7o
hniOXQ/2fUc6JswxD9S1B9qfiOn9PB5s/vntg9+ZkN3sQmBpMHXH7wTgUmTK
94sLkctWGx8S/8FhZesc/vF34tJo87DB8oT/d22XyGet+cc2mnwWeB7ewNA9
HCewdewcnyKnpvahCfYbMATz5NnLXQPn3Je8BvskD1JnJxfyu8xIv7u3mx/k
v99rqn3uWOtincdgNt7F1H/BPMh8kmW432Pog6PgFd8nLqOLbtbL5X5HI6sb
TB958gYAE8yy/BtK/r3UazZ5PNt20sNypmZEfWNJHnI+x2vareu7jDeo2eGb
xK/LbFfttmHo8HbpapkfNT/onRrGcMdusBi2gj8dNv89LbfupnPYZ822PXS3
PNvNM7GF+uVm163A/rwBwA8bHQt4C/D7Gb/9zJUdbipEfYA6wcJi1Hnn56Ez
7Kmb+WeMXX1hmwe/IZOpth9+V7zSv+fyO8Uw2zy/XTT4twPeWeRj4jCxh9//
iD/gLeY7cjfv3bYkdM79kTNxCOxIPAeTgn2JG909bjYePtAx773Ih7rvdteu
kUujfQG8Q50HX5yXxxvm/jxiSqPviD54q2eea/LeJf69mnw+3PPsW5rHb2nL
8sAV5xp/7LV93Gm80WybR7bgG+4LNj/ou4ADH/U74kwOcq5r8nsfHyyWokY2
pxRYe6pj0eRC1LhXlQOvTumIS3WBw4gtvEPXGI+dMD/48knzjE7BrvAABq00
D8QTcDI1R7AycXSv7wWdPrZDfj8d7xrycc+Tn5+xjtDbjlK8x3irtRkH3lqO
uVHGXfjNO7a9DltEVv8H/5juZg==
           "]], 
          PolygonBox[CompressedData["
1:eJwllVtslUUUheec056e8v//OdNoUQRj1MRQkQoqYikihpRKU4ghwaRUDRZe
TAQT30xbWykmqNhYFamXF9FEXxQaH8TWAloqFEEpF9PyYjTeEjVBHjTRRPxW
1sNO9p59mT0za6+5sevJjTvyIYQ+pAoZq4TwU00IywshjKI3lUPI5UJoK4aw
An19ZA3fC6UQNqCX0xCO41uJ3Ir/RewmCjUjH2bEEXOq5DX53kba0N+oDmEf
vvPsUcQ/jP0s8Rewa7A/p94U9V4lZi++VuWQu4WYdvq8ibXH0K+O7nUjOfXo
c4hZj96OROyI/SD6GPUmqHcKaSF+Evtr9EeSEBqodYT6l4idImcz9gj2z9g/
IvW1IfzGnRyht3PknCZ3FbIY/QLSnfPat+iX6CmPHuSn9lnyA/lX6OETalew
T+C7jfp96I8jEf1T/D34d5HTzP4z2HsVj7SVvCbf9did9J9HLmL3pM49hHTj
m4v9EPomZG50j+pVe2rv+ax14NtM/jzsrfS7OzjnGnyHOWNHwTELsN8i5goA
KXHGN9FrWFuNb4p6Bexd2BPYjZxhJ/qj9NTKXd2OPYC/mrWtBWPmP3z/INPE
r8U/jL+B/grUaqT+QvRZ1oar/IZ6S2FK2LoXWYS+h7Wv8J1EdlP75WhsnMd+
JXpNPmHuJWK3c57F+LfT0w70XtZuwb6Z++wldpKa99HbYeofR3+q4rdfN4eY
Os6Q+K105/PJ/TM1Vo6Sf4b8jJwF5A4gKfoI+WdqfGcH0d/PfHcLyfkA/cuy
93qHmDuo3R49S1uIaaH2utRvez+Y+R7fDNKAvgipJr+JnKHgO3ge3zHqfVb0
DGmWVFO11dPD6BOsraHeZdaOVbyH9tJM9WY+o86qO+oXlqKxmiPnEHofMq67
xL+Ees9En0Vr/ejPJa4tjAqry7D313imNFsjme9KM63ZFkaF1U2coZv9vqD/
0aLPkMMu0c9krWf0V+yh6LddoX6pd11q7IpzxD3j1GwuGLPCbmtqLhmkx2n0
UfLztb4z3V1efFEyBoXFu5F+fGuI+QFfVWruEcaF9V+wl+Xck3obSozdFmQf
vsHUXCfMzqB/U/ZsiHPEPUVqrCqZU3vIXY7cUPSMaFbuTNzr0/TcxV2dIH9b
wZjMk/9e5tlSD++iz0bPijBxMfpN9DbiHHHPzszcKs4R94zhPw0W76GHAvFH
K+YycXqffGVzod5Mb3dZHJx3jxV8/ybO1Z3p7l6P5iZx+GvRb6K30Rl11icq
njXd+Tn230a9zmpzjLjmj8zcqJ5+z4xpYVuc25z5DnQXo5x/KbVnE2NxkJi/
tTd2wN6D/Rd2SM3twpCwpBzlagY7E/8B+gs2IAmxZ8vmEnH2NHpdZi4Qx4vr
hQFhQZx0Ev+Biv++Lrb6qOKZ1GwKM3cJO2XPvnpSbx2Jsf0xdj21vyN+Za0x
ujZ1z+pdnLiE3Kui/y79afX4V6fOFUc9EH1mnV2YWEr8vGhu10xfi14XfTa9
2Th7NybuVXtqb3GwuFgcPYC9PzN29efo7/kf0zL3CQ==
           "]]}]}, {
        EdgeForm[], 
        RGBColor[0.35225, 0.4275, 0.612], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNm3m4VMW1xbv79u3u2+d097mIUUM0QZOnYPI+BuOAoCYgiMYhxggmIqiJ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           "]], 
          PolygonBox[CompressedData["
1:eJwtmHmQVdURxu/Me/PmDfe+e++ACEqiAa0Clywa0WEYhx2HsMaYgEhKkliV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           "]]}]}, {
        EdgeForm[], 
        RGBColor[
        0.5823970588235293, 0.5083823529411764, 0.47670588235294126`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxFmnm8llW1x5/DOeednvd93gcxBzC9il5NLet2UdNUBssEGwQcAEvEiVEM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           "]], 
          PolygonBox[CompressedData["
1:eJwtmHl0VdUVxi95yXsJ9717Ly0ig4qMVlCXUwEZVAK2lkEKaBFQGRxKEjC0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           "]]}]}, {
        EdgeForm[], 
        RGBColor[
        0.8125441176470588, 0.5892647058823529, 0.34141176470588236`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNmXmQlcUVxb8wzLw38977vofAgOsAwgwDxYCIoknURBNDgkQFERdEFtmR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           "]], 
          PolygonBox[CompressedData["
1:eJwll2dsllUUx6+Utujzvvd9UWgRtKUsaVmyjQMRNCIIKLTKkl1bAVtAP/hN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           "]]}]}, {
        EdgeForm[], 
        RGBColor[0.9227941176470588, 0.676985294117647, 0.36066176470588224`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNmnm01dV1xy/w7rvc+xuf1TiAypyKICgaM2i0jUqS1TCIYsWoDCq2VQQU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           "]], 
          PolygonBox[CompressedData["
1:eJwlmGl0VdUVx294SV7KfXcAcUBRwSIINcyoq0KlBQK0tYyFVkaZlxWS0CKs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           "]]}]}, {
        EdgeForm[], 
        RGBColor[
        0.9595588235294118, 0.7688970588235293, 0.47463235294117634`], 
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxNmgm0VeV1xw/33cs9991z7jkPTASioFEkaWJsAw5EwQiIS0BsXhRNBVxR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           "]], 
          PolygonBox[CompressedData["
1:eJwtl2lwVuUVx2+S982beO99703AEUFQlpC6AJ0iKFJENh0ZAUmRpZYysjlU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           "]]}]}, {
        EdgeForm[], 
        RGBColor[0.9963235294117647, 0.8608088235294117, 0.5886029411764704], 
        
        GraphicsGroupBox[{
          PolygonBox[CompressedData["
1:eJxFVl1sVEUUnt3uWnbvnXtX7BY0VYtQETEYJSxvtgs+gLCybf1pTCRKIlYh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           "]], 
          PolygonBox[CompressedData["
1:eJwtlF1Mz2EUxx//+kv9/k//YixbKEoqYxjuqK4okRo1G8NmYokmFyluuXPj
JUXaaOllarY2ZbNq5mUKm0xlIzbmghtmY+bl83W6ONv3nPM95zzPOed50vcf
La0OOefKkVikzzv3IOzcXJQjEecmo84lznLOIxPgQ/hPxmCDWwm+inwCp8Fv
AaclO7c+wbl9cMbhB9jq44hB6tHvkHMneAdSjp4cODfTGachajVVW5wK9MvE
d3KeKTgn4A7gr8S/Dlt3knON+DvAE/h3g0uIKSG2FOmE24btcMju8BrfdkT3
/c8BV8OZR74ipBB9AzLuzLYR3E/N+wSkxjs3Sr0+bJPEruLQeeCF5D+Dv4j7
LgJnImXg09gy1Cs4dfA/q5/oNdTr4Sy3kOPku0n+syGz1aIP0dga4tcgPwPr
uXo/hVwnNt1brYPEpIEvwdkCLsWWjd6Evi1kZ8hFzyFmBXor5/9F/huBnW2v
auBf7m1WimnGNwgnAW4EeUtsjjeuajTibw2stnqyEl8KnAr8+czgOWc/xXkb
uG8KegL+ISQbbkC+IXK/iRhWDdXqJd/F0PRM4X6JWK//kOMJuSLohTG2U9qt
gUTrvWbUD+5Ksl0aQ+8EdxFzhVzF8DPgF6DHo7fTvyr8g8Rs5myryfmD2qkR
w1mc4UJgPVKvdKdcfGs5055Ym3E7/rKo7dYubB34l3jrve6w2NuOa9fVgwh6
G/l+x1qOHvj53nqD2+WB7+rNhO1NDYCbk2zW2oFM+C+wfQjbGziA3oz+PdZs
Y952SLukHT+G/zG2V2Gr2YU+nGSz1I5p14qjtlv5+LdG7U3pbalHC4h9FJ1+
C3AegmsDe1vaee3+KPI3zmaq2b5EPoatp9fgPkN/B85DloHn6I8g3wj6PW89
U++G0W97m7FmrZwj4Ca94Rl2R9011dtbf4oMgluosSlsNVX7fGC7p5qqrZlr
9nnEn6NWrzesnD3grES7+3xkaaL9YfrLvhH/3tsb1VvVn1blraZq60/sJ193
xM6uHqvXBd5m+VV/lO7uLZfefJ3errda+uP018329leqJ+rNPzXytDg=
           "]]}]}, {
        EdgeForm[], 
        RGBColor[1., 0.95, 0.75], 
        GraphicsGroupBox[{
          
          PolygonBox[{{2975, 596, 3541}, {893, 1757, 15}, {2011, 1743, 862}, {
           1757, 2136, 1738}, {3535, 510, 2964}, {3541, 596, 2974}, {4347, 
           893, 3114}, {2965, 862, 3100}, {3064, 1720, 3776}, {2012, 1743, 
           225}, {3368, 510, 3535}, {3115, 893, 4347}, {1738, 1739, 510}, {
           1757, 1738, 15}, {1743, 2012, 1586}, {2136, 1739, 1738}, {3792, 
           1739, 3096}, {1720, 1586, 596}, {1743, 2011, 225}}], 
          
          PolygonBox[{{3095, 1757, 893, 3115}, {3096, 1739, 2136, 4122}, {
           3776, 1720, 596, 2975}, {2964, 510, 1739, 3792}, {4122, 2136, 1757,
            3095}, {3619, 1586, 1720, 3064}, {3100, 862, 1743, 3799}, {3799, 
           1743, 1586, 3619}}]}]}}, {{}, 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[{2965, 3100, 3799, 3619, 3064, 3776, 2975, 3541, 2974}], 
          
          LineBox[{3114, 4347, 3115, 3095, 4122, 3096, 3792, 2964, 3535, 
           3368}]}, "5"], Annotation[#, 5, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          
          LineBox[{2836, 3342, 3515, 3514, 3112, 3433, 3432, 3554, 3553, 3961,
            3962, 3437, 3436, 2937, 3485, 3486, 3191, 3545, 3346, 3347, 3518, 
           3517}], 
          
          LineBox[{2893, 2931, 3479, 4266, 4267, 3927, 3256, 3928, 3181, 3509,
            3257, 3520, 3356, 4268, 3258, 4069, 4070, 3382, 2992, 3532, 3362, 
           3363, 3089, 3369, 2967, 3536, 2968, 3370, 3630, 3379, 2980, 3533, 
           3364, 3365, 3090, 3371, 2970, 3537, 2971, 3372, 2969, 4056, 4384, 
           3084, 4114, 4039, 4040, 3229, 3914, 3182, 3913, 3183, 3903, 3902, 
           3769, 3770, 3618, 3904, 4021, 2932, 3905, 3421, 3480, 3170}], 
          
          LineBox[{2896, 4011, 4388, 3215, 4237, 3216, 3504, 3503, 2958, 3482,
            3483, 3540, 3539, 3173, 3481, 2933, 3357, 3174, 3906, 4204, 
           4203}], 
          LineBox[CompressedData["
1:eJwVzT1OQmEQheGxFa5o7EUSwNLKWqUGg7gDC0rAn05dhLWNiRoUjCZ2dF47
QBaAJsIGXACdj8Wb9845M98tHLcbraWIqKKZROwil4k4zEU0UEcvG3HET/ym
f+Yh9/lhNeKV7/mFd+Rd9yPeQB6JvMCbGONWv80Dnrj71Y+9/4kJUl0q/5G/
84w/+NH+OZ+Yz7jz/3+7326+MMWd7EZX4nQlYuGmyDXdAa7lW+YySjg1J95Y
s3epz/i+4Cxf8TLvYw/rdio8xwx/fIcrXg==
           "]]}, "4"], 
        Annotation[#, 4, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwV0EtL1GEUx/EjLYqu5syiCKI2ubIg26S2lkoSpMxLNGS6UGiEQH0RVt4o
WtTS8YK1KFNBrZVUKi7mJVhQuiydKdPJzyy+fOd3znnO8/znfFt3Q7okIp7h
5pGI8RMRV1GNH6URVXzmWMRb/NK/xy/wHI/lGxgz809+cDQixfdRwB5u693B
5eMRlXhjPiVPcS/3oQfbzn4x/7l4l15OPnwyIs/vMI0D8nvuMJPkVe/b5BXe
4CX173Z9wzq21LJ6l9x7Ue+1vVl+xRWc0C/nC1hDRv0Kj3LB+f/F78IedvHS
rqd6y35/xRO5Hw/t6eR27uK8mXPuzPEnnjT7kbfleV7EAurlWZ7DB8ygzDcm
kcAt/Tpnr/OOd/1FqzsGeQAj6NY7zYfQjCYMI61+ig/iLhoxhEfqtRi194/c
Yt+4N08gg1l5jOvM/ORGc9PetSWf5d9co3at+J84P8H7rrdYvA==
           "]], 
          LineBox[CompressedData["
1:eJwV0E1LlGEUh/EzJm58GwUzgl40M/sIkq58o2XtqlURDBToohxd2TfoEwRh
ReBCIyjH2mSZNqCYNlkLHVPHBC1qpYVR9pvFxXXO/znnfu7nabjWd7E3ERFj
OFUR8aUqYh1reFUecVAZsaxewSryGDU3nIx4iAdo0ufNrSJpp4MPcyd3oQw9
6Ea9/HJ1xKSdS/yar/BVZNXPPM+Ym8AffXlNxD4fkYesUp/gA9khrtKXcMbz
v7IK/T+exbK71covIGVmW7+DJXzGJ5TKc/wRSbvV+Kb+jg3feg5tSKCATTQ7
u50nnVvCX9Fk/ox8yHfcQb8+jXF1Bmmzd/UD/EI/gYQ+cBrN+GF/Xv4eC3jj
bm/la/IpXudpHnHGIN/WD/Atflz8/3byWMEj2T3PXpqd55/6RfkH5DBl/l3x
G+UzXOAsPzW/y7/1g+o092Mcpf7Nc261e5z3zZ3gYzipbkQDnti77h2d6EIH
etCNVPFe9m8gb/Ymz8kKvIk6u718lLf4fPGe6vuc5T3/+hd2cdZ+C/4DjSRr
Lw==
           "]]}, "3"], Annotation[#, 3, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwV0TkvhFEYhuGjFHtDbBFq+97QWKIW02AQsRVia/wJVBI6+85YRk2DP6G3
DAYVUbmmuHO/z3uec75Mpnx0tncmLYSwiM6MEDZzQhjPDWEMNeaS7BD2sItK
eVqnAxvmn6wQqrkK33atPJIZwpS7wzzPC5jFtDzHD5zQe8ULKuR277ThWo7j
D1/e+9Vv9N1neQj3ulG+45izVXfm9Yo4HQPoR4vOnH1Sp9lc7o04rtCUesv9
Rm5Amd2n7iUP2tfZRbmek/bn9h98wft2MX6XT/gMp+iRl313BY+p361369s3
WLIrlad0SvJgzndnHWsolCedFehF3C2S+7iYI3bdzo70jnGAN/mQd5xPeH8S
2+Yn3uJalDpP6O2n/je5y/wPg/BCyg==
           "]], 
          LineBox[CompressedData["
1:eJwNz7kv5WEUh/FXMI0xKBTCJGQiQqxjrezLpbBcrUKIpRlEpRzL+GuILW5m
SveaERK0TKa1M9ZGovApnjznfM857+/egpHp6FRSCGEJV59CuEYjdtIEHOcL
vsQ5Sj+G0MQJeQrfIDsrhKj+Vh1JD2EjM4RDe2MZIRzxjGwaeXb67PSjF1EM
4LO8x/xe/YD/KHP3zTtTWPHOHQ/auzNrRolvtnAqHvGEcjetvGvvAz/ju9s3
txVmRTyvb5MXuv9tL1n9h9t5wWwRneoudKAb/9xFOMtNjt0lO3HvrcoTPKQf
xp56yzzmv2zyq35Ofuo2X33CBfyXR+X76pi9X/a3+U2/LP+BW9zgp/zEvB4N
OLBT774OtVj3G9YwYvfMvNL8RV7M47IJXMgvkeutWf7CE3YmMY5jfbU3alCJ
r6jCOwbLS7g=
           "]]}, "2"], Annotation[#, 2, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwVz8kuA2AUxfHPGsVCW0SrNfMGEuMLoIayQFuJBIl6C8XKUAla41tZq+EB
FBvx6+Kfc+859375bqpQzBw0hRCKmG4O4SwSwjk6EW8LodAewlJLCFX1HS75
NXMX9AplzOiXW0NYNZvFChbsRHl5WcTekbkS2lHUxzu8LTtUH5s7wYudmt1J
/ql+ltbl3/jCL34wx7/xzhu9pu80yc/aT9BVesv/4MdoHy+FfqRR4VUb92FA
3+svu2YHGzfy7tGFIX1CticbVj/wHtGNEX1Sti8bVT/xntGDMf2fO9LyFNb8
Z1u/Tsdln3aiNIa6Oyt0ireDTXNb2EAeObzyF2kGl2bLmODlULI/791/5Rg2
lA==
           "]], 
          LineBox[CompressedData["
1:eJwNzrcux3EUhvFjItEXI/6CRI8+uAM10XuX6CJI3IGyKPdh0e5BN7HZEaxq
4jM8ec55z/vL75uYXOlcToqIJbynR3xgE6WpEYeZEQc4z4o4Q1NaxAmfYk++
jxK9Df1CfuUXzMlnUa3/mxHRyM/yXVkb3+o+8CNa0YIC2bTuDCYwhUkk5GM8
jmGMYgT58kEeQh8G0I88+Y7/9JpvzN18zcXZET3mbbcq72/wpj/esv/IU3Cl
18X1bpVuvfYKPtJ5xxvK7T3yXDyZ63Q7vL8dxbJF3y9hDguYR5F8zX0d9/rH
vrvjVfu3e7d7jvd9mZNxaW/jGp1S3Vou4wt5s7wVLfjEP2hXOc0=
           "]]}, "1"], 
        Annotation[#, 1, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          LineBox[CompressedData["
1:eJwV1EdsD1AcwPFXtNWFGhEkRBwQDkhQ1B6xVUnMRGIkoknt0Ra1WrQ1orag
iFkH66YICQcnW0LRGicSFEfq8w5f3996v/f8+2+7Llyem58QQhjin+y0EA63
DGFmqxBm4IN8R4sQSlGCIfLcjBCSzCQjESlojvFYlh7CN/x2ttJsjdkX8hL8
VNutVq1Wxg3yUvWX+CzOUivlXXE+zvJO3slXnSnXLxNPcs92b6mVb+N3nGNu
kt40nhjvN/Ne/QbfVlsQ38Il9rzSvyN+HffLS3HU3BGU2VdndjfX8xxzczEb
082Xq39Uz+SW5jtkhrBI3kpcobYHrZEp76i3WK+1eK/aPrSJyDvpLdFrK861
d7/6p5hzO7V57puPY+LjOIEZ3pnOU8yn8VTO4DxO5cn8y8xv/MF0bMeOiN5D
M4+wyz2P4+fKVXxe7zRf4BoketttvoeLaCK/FHcjRdzA/ezsi1pcl9/iavta
2JMS38cZSEc3HNZ77e5X6CLvjBq1k3wKder1OKf2VH6W+9j5Fm9wzc6bfEU9
VT9J3pwT45v4Did5210+iwTxGd7izGYUo0p+zOxpPs7b7LqsvpUfuPs+vuht
VCtCASaaG6U2gUfH7x4XmW9qdmj8bDibx6qvMz+KB8tH8ph4Bi/Mb/Jzfc6b
43ckfh9QJN6IDShEAXqYWcfrsRprsQbd1VfyKizHM3l3/8cV4gq7yjHCXcOR
IE5Vf2omn7O8J6jNjt8vbxwkzzY3DEORo9Zb76eZBnyPPyd5D/t/iHuJK3m4
c+PMNxOn2ftSvZiHqb9DsvlaPmPfUi40W8gFXMD7+GD8HXX+EFeiUe0AVzlT
aN93+YY4K36ktp9/qX1VW+3cGvxzZ093NfLf+DcpfuZYbz5Pf5Yz7fWXxb9N
+k/kWfqD0B8DMQD/AVPdpbM=
           "]]}, "0"], 
        Annotation[#, 0, "Tooltip"]& ], 
       TagBox[
        TooltipBox[{
          Directive[
           GrayLevel[0], 
           Opacity[0.5]], 
          
          LineBox[{3006, 3388, 3428, 3427, 4119, 3094, 4118, 4121, 4120, 3079,
            4035, 4036, 2949, 4111, 2948, 4245, 4246, 4244, 3228, 4247, 3144, 
           3458, 3282, 4279, 4281, 4280, 3598, 3713, 3033, 4343, 3633, 4063, 
           3784, 3074, 3781, 3783, 3782, 2866, 2908, 3597, 3596, 3712, 3032, 
           4342, 3632, 2977, 4101, 3070, 4100, 3069, 4173, 4174, 4172, 3141, 
           3453, 3140, 4243, 3225, 4240, 4242, 4241, 2941, 4102, 2940, 4027, 
           4026, 3068, 4116, 4117, 4115, 3093, 3424, 3425, 3344, 3343}], 
          LineBox[CompressedData["
1:eJwl0jtszWEYx/GnLSGqPecMHSiDWNzqkrq1Fhat1YYwdGtCJUa9nHN6RFxK
woCQGJAQidK1FxW6FEVVtdVBXYJBk5YJg37+6fDN93l+7/O85z05Z1XD8f1N
RRHRjT1LI+rSEbu5nlu5lTeVRmTKIr7q+1MRT5CVDcjauKY8ohY78dBZJw7K
N+Kj/b32/qt75X14jBXySixz9oeXcznWuWM9NuAthnHZ7ii/xzuMoAop81nM
2C+YqVSvVH9Q//YZk7xP9km2Vf3Cznb+rH+unvX+tbwGp+X9+olkH+NYqN+F
Wgzqv7nrhHtXZyJy6rwsz8dkTTiKEX23vMZ9Z7lDf4Yv8Dk+zw/c1+H8onoc
E1iCtPNSznCf81/mUt72Rb9DfwQF5wX9KbQn6O+Yu437+K5f7H0/EuOl/VcY
wjb7hzEjf213lt/wTe6yO+r9g+YW6BfJS5I5fZqfcfBTLuKB5Lcwn7dXpx/G
lJ1q9zfItvAhdJl5hDH1DTPXkVPfNXMv+f/o2+zlkMXm5H3yKp7mn+hRlzj7
x8X8l6/Zu8RXOXzfK3zLbCO3mGnhZm7mk/IDZivK5v/jcxb4a4Q=
           "]]}, 
         RowBox[{"-", "1"}]], Annotation[#, -1, "Tooltip"]& ], {}, {}}}], {
    DisplayFunction -> Identity, AspectRatio -> 1, DisplayFunction :> 
     Identity, Frame -> True, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], Method -> {"DefaultBoundaryStyle" -> Automatic}, 
     PlotRange -> {{0, 1}, {0, 1}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
       FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           GraphicsBox[{
             
             RasterBox[{{{0.148, 0.33, 0.54}}, {{0.35225, 0.4275, 0.612}}, {{
              0.5823970588235293, 0.5083823529411764, 
              0.47670588235294126`}}, {{0.8125441176470588, 
              0.5892647058823529, 0.34141176470588236`}}, {{
              0.9227941176470588, 0.676985294117647, 
              0.36066176470588224`}}, {{0.9595588235294118, 
              0.7688970588235292, 0.4746323529411762}}, {{0.9963235294117647, 
              0.8608088235294117, 0.5886029411764704}}, {{1., 0.95, 0.75}}}, {{
                Rational[-15, 2], -100}, {
                Rational[15, 2], 100}}], {Antialiasing -> False, 
              AbsoluteThickness[0.1], 
              Directive[
               Opacity[0.3], 
               GrayLevel[0]], 
              LineBox[
               NCache[{{
                  Rational[15, 2], -100}, {
                  Rational[-15, 2], -100}, {
                  Rational[-15, 2], 100}, {
                  Rational[15, 2], 100}}, {{
                 7.5, -100}, {-7.5, -100}, {-7.5, 100}, {7.5, 100}}]]}, {
              CapForm[None], {{
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], -75.}, {
                    Rational[15, 2], -75.}}, {{-7.5, -75.}, {7.5, -75.}}]]}, {
               
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], -50.00000000000001}, {
                    Rational[
                    15, 2], -50.00000000000001}}, {{-7.5, \
-50.00000000000001}, {7.5, -50.00000000000001}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], -25.000000000000004`}, {
                    Rational[
                    15, 2], -25.000000000000004`}}, {{-7.5, \
-25.000000000000004`}, {7.5, -25.000000000000004`}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 0.}, {
                    Rational[15, 2], 0.}}, {{-7.5, 0.}, {7.5, 0.}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 24.999999999999993`}, {
                    Rational[15, 2], 24.999999999999993`}}, {{-7.5, 
                  24.999999999999993`}, {7.5, 24.999999999999993`}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 49.99999999999999}, {
                    Rational[15, 2], 49.99999999999999}}, {{-7.5, 
                  49.99999999999999}, {7.5, 49.99999999999999}}]]}, {
                Directive[
                 GrayLevel[0], 
                 Opacity[0.5]], 
                CapForm["Butt"], 
                LineBox[
                 NCache[{{
                    Rational[-15, 2], 75.}, {
                    Rational[15, 2], 75.}}, {{-7.5, 75.}, {7.5, 75.}}]]}}}, {
             Antialiasing -> False, 
              StyleBox[
               LineBox[{{7.5, -100.}, {7.5, 100.}}], 
               Directive[
                AbsoluteThickness[0.2], 
                Opacity[0.3], 
                GrayLevel[0]], StripOnInput -> False], 
              StyleBox[
               StyleBox[{{
                  StyleBox[
                   LineBox[{{{7.5, -75.}, 
                    Offset[{4., 0}, {7.5, -75.}]}, {{
                    7.5, -25.000000000000004`}, 
                    Offset[{4., 0}, {7.5, -25.000000000000004`}]}, {{7.5, 
                    24.999999999999996`}, 
                    Offset[{4., 0}, {7.5, 24.999999999999996`}]}, {{7.5, 
                    74.99999999999999}, 
                    Offset[{4., 0}, {7.5, 74.99999999999999}]}}], 
                   Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], StripOnInput -> False]}, 
                 StyleBox[
                  StyleBox[{{
                    StyleBox[{
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "1"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -75.}], {-1, 0.}, Automatic, {1, 
                    0}], 
                    InsetBox[
                    FormBox["1", TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -25.000000000000004`}], {-1, 0.}, 
                    Automatic, {1, 0}], 
                    InsetBox[
                    FormBox["3", TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 24.999999999999996`}], {-1, 0.}, 
                    Automatic, {1, 0}], 
                    InsetBox[
                    FormBox["5", TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 74.99999999999999}], {-1, 0.}, 
                    Automatic, {1, 0}]}, 
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], {
                    Directive[
                    Opacity[1]]}, StripOnInput -> False]}, {}}, {
                    Directive[
                    Opacity[1]]}, StripOnInput -> False], "GraphicsLabel", 
                  StripOnInput -> False]}, "GraphicsTicks", StripOnInput -> 
                False], 
               Directive[
                AbsoluteThickness[0.2], 
                Opacity[0.3], 
                GrayLevel[0]], StripOnInput -> False]}}, PlotRangePadding -> 
            Scaled[0.02], PlotRange -> All, Frame -> True, 
            FrameTicks -> {{False, False}, {True, False}}, FrameStyle -> 
            Opacity[0], FrameTicksStyle -> Opacity[0], 
            ImageSize -> {Automatic, 200}, BaseStyle -> {}], Alignment -> 
           Left, AppearanceElements -> None, ImageMargins -> {{5, 5}, {5, 5}},
            ImageSizeAction -> "ResizeToFit"], LineIndent -> 0, StripOnInput -> 
          False], {FontFamily -> "Arial"}, Background -> Automatic, 
         StripOnInput -> False], TraditionalForm]}, "BarLegend", 
      DisplayFunction -> (#& ), 
      InterpretationFunction :> (RowBox[{"BarLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{
                 RowBox[{"Blend", "[", 
                   RowBox[{"\"M10DefaultDensityGradient\"", ",", "#1"}], 
                   "]"}], "&"}], ",", 
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "1.52`"}], ",", "5.280000000000001`"}], 
                 "}"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"-", "1.`"}], ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"0.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"1.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"2.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"3.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"4.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                 RowBox[{"5.`", ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    RowBox[{
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, AspectRatio -> 1, Frame -> 
                    True, FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "\[InvisibleSpace]"}], "GrayLevel[0]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                    ",", 
                    RowBox[{"Opacity", "[", "0.5`", "]"}]}], "]"}]}], "}"}]}],
              "}"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
           RowBox[{"Charting`TickAnnotations", "\[Rule]", "None"}], ",", 
           RowBox[{"Charting`TickSide", "\[Rule]", "Right"}], ",", 
           RowBox[{"ColorFunctionScaling", "\[Rule]", "True"}]}], "]"}]& )], 
     TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.6157440881894817`*^9, {3.6157441263636656`*^9, 3.6157441715542502`*^9}, {
   3.615744288841959*^9, 3.6157444267658477`*^9}, {3.6157453900039415`*^9, 
   3.615745403069689*^9}, {3.6157455784037175`*^9, 3.6157456033601446`*^9}, 
   3.616166760912716*^9, 3.61617423978603*^9, {3.6161840929595995`*^9, 
   3.616184299856433*^9}, 3.6161865482720346`*^9, 3.6162655651184607`*^9, 
   3.6162667551227627`*^9}]
}, Open  ]],

Cell[TextData[{
 "You can use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function CForm to convert the function we defined in mathematic into C++ \
appropriate syntax."
}], "Text",
 CellChangeTimes->{{3.616266740209137*^9, 3.6162667707559905`*^9}, {
  3.6162668086026573`*^9, 3.616266888164797*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CForm", "[", 
  RowBox[{"g", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.615743346134039*^9, 3.6157433544815164`*^9}}],

Cell["\<\
20*(-0.9 + x)*(-0.4 + x)*(-0.1 + x) + 20.*(-0.7 + y)*(-0.3 + y)\
\>", "Output",
 CellChangeTimes->{{3.615743354809535*^9, 3.615743370120411*^9}, 
   3.615744533346944*^9, 3.6157456063013134`*^9, 3.616184301919551*^9, 
   3.6161862744573736`*^9, 3.616266760396572*^9, 3.6162668910820017`*^9}]
}, Open  ]],

Cell[TextData[{
 "We can also combine the ",
 StyleBox["CForm", "Input"],
 " function with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s function for calculating partial derivatives, ",
 StyleBox["D", "Input"],
 ".\nThen we can simply copy the below expressions into the ",
 StyleBox["derivative", "Output"],
 " function in the CBasicPolynomial.cpp file."
}], "Text",
 CellChangeTimes->{{3.616266897322013*^9, 3.6162669986199913`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CForm", "[", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", "x"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.616184326117935*^9, 3.6161843287830877`*^9}, 
   3.616186396935379*^9, 3.6162655537238092`*^9, {3.616266790865426*^9, 
   3.616266800381443*^9}}],

Cell["\<\
20*(-0.9 + x)*(-0.4 + x) + 20*(-0.9 + x)*(-0.1 + x) + 20*(-0.4 + x)*(-0.1 + x)\
\>", "Output",
 CellChangeTimes->{
  3.6161843292891164`*^9, 3.6161845580762024`*^9, 3.6161862812167606`*^9, 
   3.6161863978344307`*^9, 3.616265555824929*^9, 3.6162667623007755`*^9, {
   3.616266793579831*^9, 3.616266800958644*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CForm", "[", 
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"g", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", "y"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.6161843362895164`*^9, 3.616184336709541*^9}}],

Cell["20.*(-0.7 + y) + 20.*(-0.3 + y)", "Output",
 CellChangeTimes->{{3.616184334284402*^9, 3.6161843371425657`*^9}, 
   3.61618456013132*^9, 3.616186283261877*^9, 3.6162667638451786`*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1902, 980},
WindowMargins->{{-9, Automatic}, {Automatic, 0}},
Magnification:>1.4 Inherited,
FrontEndVersion->"10.0 for Microsoft Windows (64-bit) (July 1, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 163, 2, 96, "Section"],
Cell[745, 26, 229, 5, 42, "Text"],
Cell[977, 33, 1088, 30, 42, "Input"],
Cell[2068, 65, 590, 12, 71, "Text"],
Cell[CellGroupData[{
Cell[2683, 81, 600, 13, 42, "Input"],
Cell[3286, 96, 183272, 3156, 523, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[186607, 3258, 197, 2, 96, "Section"],
Cell[186807, 3262, 517, 8, 70, "Text"],
Cell[187327, 3272, 574, 15, 124, "Input"],
Cell[187904, 3289, 448, 8, 70, "Text"],
Cell[CellGroupData[{
Cell[188377, 3301, 660, 18, 42, "Input"],
Cell[189040, 3321, 1911, 39, 658, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[191000, 3366, 179, 2, 96, "Section"],
Cell[CellGroupData[{
Cell[191204, 3372, 176, 2, 67, "Subsection"],
Cell[191383, 3376, 313, 7, 43, "Text"],
Cell[191699, 3385, 165, 4, 42, "Input"],
Cell[191867, 3391, 1164, 19, 152, "Text"],
Cell[CellGroupData[{
Cell[193056, 3414, 389, 9, 70, "Input"],
Cell[193448, 3425, 796, 28, 41, "Output"],
Cell[194247, 3455, 300, 9, 41, "Output"]
}, Open  ]],
Cell[194562, 3467, 257, 5, 70, "Text"],
Cell[CellGroupData[{
Cell[194844, 3476, 1258, 38, 205, "Input"],
Cell[196105, 3516, 2391, 48, 324, "Output"],
Cell[198499, 3566, 2315, 46, 324, "Output"]
}, Open  ]],
Cell[200829, 3615, 370, 6, 42, "Text"],
Cell[201202, 3623, 606, 17, 42, "Input"],
Cell[201811, 3642, 6137, 128, 556, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[207985, 3775, 176, 2, 67, "Subsection"],
Cell[208164, 3779, 585, 11, 71, "Text"],
Cell[CellGroupData[{
Cell[208774, 3794, 441, 10, 70, "Input"],
Cell[209218, 3806, 650, 22, 46, "Output"],
Cell[209871, 3830, 313, 11, 41, "Output"]
}, Open  ]],
Cell[210199, 3844, 371, 8, 70, "Text"],
Cell[CellGroupData[{
Cell[210595, 3856, 1091, 32, 205, "Input"],
Cell[211689, 3890, 2160, 45, 331, "Output"],
Cell[213852, 3937, 2087, 44, 331, "Output"]
}, Open  ]],
Cell[215954, 3984, 502, 8, 42, "Text"],
Cell[216459, 3994, 950, 23, 70, "Input"],
Cell[217412, 4019, 6206, 128, 612, "Input"],
Cell[223621, 4149, 94, 1, 42, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[223764, 4156, 270, 3, 96, "Section"],
Cell[CellGroupData[{
Cell[224059, 4163, 1328, 34, 97, "Input"],
Cell[225390, 4199, 134035, 2375, 523, "Output"]
}, Open  ]],
Cell[359440, 6577, 315, 8, 43, "Text"],
Cell[CellGroupData[{
Cell[359780, 6589, 178, 4, 42, "Input"],
Cell[359961, 6595, 301, 5, 41, "Output"]
}, Open  ]],
Cell[360277, 6603, 463, 12, 72, "Text"],
Cell[CellGroupData[{
Cell[360765, 6619, 334, 8, 58, "Input"],
Cell[361102, 6629, 323, 6, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361462, 6640, 234, 6, 58, "Input"],
Cell[361699, 6648, 188, 2, 57, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
